package com.mt.wms.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.basic.params.AlarmBaseParam;
import com.mt.wms.basic.params.AlarmBaseQueryParam;
import com.mt.wms.basic.service.AlarmBaseService;
import com.mt.wms.basic.vo.AlarmBaseVo;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.AlarmBase;
import com.mt.wms.core.dal.entity.Warehouse;
import com.mt.wms.core.dal.service.AlarmBaseServiceBiz;
import com.mt.wms.core.errorcode.ApiErrorCode;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/10/9 10:49
 * @Version 1.0
 */
@Service
@Transactional
public class AlarmBaseServiceImpl extends BaseService implements AlarmBaseService {

    @Resource
    private AlarmBaseServiceBiz alarmBaseServiceBiz;

    @Override
    public R<AlarmBaseVo> get(IdParam idParam) {
        Assert.notNull(ApiErrorCode.INVALID_PARAMETER,idParam.getId());
        AlarmBase alarmBase = alarmBaseServiceBiz.getById(idParam.getId());
        AlarmBaseVo alarmBaseVo = AlarmBaseVo.builder().build();
        BeanUtils.copyProperties(alarmBase,alarmBaseVo);
        return successful(alarmBaseVo);
    }

    @Override
    public R<List<AlarmBaseVo>> list(AlarmBaseQueryParam alarmBaseQueryParam) {
        QueryWrapper<AlarmBase> wrapper=new QueryWrapper<>();
        wrapper.eq(AlarmBase.VALID,1);
        List<AlarmBase> alarmBaseList = alarmBaseServiceBiz.list(wrapper);
        List<AlarmBaseVo> alarmBaseVoList = com.mt.wms.core.utils.BeanUtils.copyList(alarmBaseList, AlarmBaseVo.class);
        return successful(alarmBaseVoList);
    }

    @Override
    public R<PageVo<AlarmBaseVo>> page(AlarmBaseQueryParam alarmBaseQueryParam) {
        QueryWrapper<AlarmBase> wrapper=new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(alarmBaseQueryParam.getAlarmClass()),AlarmBase.ALARM_CLASS,alarmBaseQueryParam.getAlarmClass())
                .orderByDesc(AlarmBase.CREATE_TIME);
        Page<AlarmBase> page = alarmBaseServiceBiz.page(new Page<>(alarmBaseQueryParam.getCurrent(), alarmBaseQueryParam.getSize()), wrapper);
        return successful(new PageVo<>(page,AlarmBaseVo.class));
    }

    @Override
    public R<IdVo> add(AlarmBaseParam alarmBaseParam) {
        QueryWrapper<AlarmBase> wrapper=new QueryWrapper<>();

        AlarmBase alarmBase=new AlarmBase();
        BeanUtils.copyProperties(alarmBaseParam,alarmBase);
        setCommonField(alarmBase);
        alarmBaseServiceBiz.save(alarmBase);
        return successful(IdVo.builder().id(alarmBase.getId()).build());
    }

    @Override
    public R<IdVo> update(AlarmBaseParam alarmBaseParam) {
        AlarmBase alarmBase = alarmBaseServiceBiz.getById(alarmBaseParam.getId());
        if (!alarmBaseParam.getCode().equals(alarmBase.getCode())){
            QueryWrapper<AlarmBase> wrapper=new QueryWrapper<>();
            wrapper.eq(AlarmBase.CODE,alarmBaseParam.getCode());
            wrapper.eq(AlarmBase.VALID,1);
            Assert.eqZero(alarmBaseServiceBiz.count(wrapper),"报警编码已存在！");
        }
        AlarmBase updateAlarmBase=new AlarmBase();
        BeanUtils.copyProperties(alarmBaseParam,updateAlarmBase);
        setUpdateCommonField(updateAlarmBase);
        alarmBaseServiceBiz.updateById(updateAlarmBase);
        return successful(IdVo.builder().id(updateAlarmBase.getId()).build());
    }

    @Override
    public R<IdVo> delete(IdParam idParam) {
        alarmBaseServiceBiz.removeById(idParam.getId());
        return successful(IdVo.builder().id(idParam.getId()).build());
    }
}
