package com.mt.wms.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.basic.params.AlarmBaseParam;
import com.mt.wms.basic.params.AlarmBaseQueryParam;
import com.mt.wms.basic.params.CraftInfoParam;
import com.mt.wms.basic.params.CraftInfoQueryParam;
import com.mt.wms.basic.service.AlarmBaseService;
import com.mt.wms.basic.service.CraftInfoService;
import com.mt.wms.basic.vo.AlarmBaseVo;
import com.mt.wms.basic.vo.CraftInfoVo;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.AlarmBase;
import com.mt.wms.core.dal.entity.CraftInfo;
import com.mt.wms.core.dal.service.AlarmBaseServiceBiz;
import com.mt.wms.core.dal.service.CraftInfoServiceBiz;
import com.mt.wms.core.errorcode.ApiErrorCode;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/10/9 10:49
 * @Version 1.0
 */
@Service
@Transactional
public class CraftInfoServiceImpl extends BaseService implements CraftInfoService {

    @Resource
    private CraftInfoServiceBiz craftInfoServiceBiz;

    @Override
    public R<CraftInfoVo> get(IdParam idParam) {
        Assert.notNull(ApiErrorCode.INVALID_PARAMETER,idParam.getId());
        CraftInfo craftInfo = craftInfoServiceBiz.getById(idParam.getId());
        CraftInfoVo craftInfoVo = CraftInfoVo.builder().build();
        BeanUtils.copyProperties(craftInfo,craftInfoVo);
        return successful(craftInfoVo);
    }

    @Override
    public R<List<CraftInfoVo>> list(CraftInfoQueryParam craftInfoQueryParam) {
        QueryWrapper<CraftInfo> wrapper=new QueryWrapper<>();
        wrapper.eq(CraftInfo.VALID,1);
        List<CraftInfo> craftInfoList = craftInfoServiceBiz.list(wrapper);
        List<CraftInfoVo> craftInfoVoList = com.mt.wms.core.utils.BeanUtils.copyList(craftInfoList, CraftInfoVo.class);
        return successful(craftInfoVoList);
    }

    @Override
    public R<PageVo<CraftInfoVo>> page(CraftInfoQueryParam craftInfoQueryParam) {
        QueryWrapper<CraftInfo> wrapper=new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(craftInfoQueryParam.getCraftCode()),CraftInfo.CRAFT_CODE,craftInfoQueryParam.getCraftCode())
                .orderByAsc(CraftInfo.CREATE_TIME);
        Page<CraftInfo> page = craftInfoServiceBiz.page(new Page<>(craftInfoQueryParam.getCurrent(), craftInfoQueryParam.getSize()), wrapper);
        return successful(new PageVo<>(page,CraftInfoVo.class));
    }

    @Override
    public R<IdVo> add(CraftInfoParam craftInfoParam) {
        QueryWrapper<CraftInfo> wrapper=new QueryWrapper<>();

        CraftInfo craftInfo=new CraftInfo();
        BeanUtils.copyProperties(craftInfoParam,craftInfo);
        setCommonField(craftInfo);
        craftInfoServiceBiz.save(craftInfo);
        return successful(IdVo.builder().id(craftInfo.getId()).build());
    }

    @Override
    public R<IdVo> update(CraftInfoParam craftInfoParam) {
        CraftInfo craftInfo = craftInfoServiceBiz.getById(craftInfoParam.getId());
        CraftInfo updateCraftInfo=new CraftInfo();
        BeanUtils.copyProperties(craftInfoParam,updateCraftInfo);
        setUpdateCommonField(updateCraftInfo);
        craftInfoServiceBiz.updateById(updateCraftInfo);
        return successful(IdVo.builder().id(updateCraftInfo.getId()).build());
    }

    @Override
    public R<IdVo> delete(IdParam idParam) {
        craftInfoServiceBiz.removeById(idParam.getId());
        return successful(IdVo.builder().id(idParam.getId()).build());
    }
}
