package com.mt.wms.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.basic.params.LocationInfoParam;
import com.mt.wms.basic.params.LocationInfoQueryParam;
import com.mt.wms.basic.service.LocationInfoService;
import com.mt.wms.basic.vo.LocationInfoVo;
import com.mt.wms.basic.vo.VehicleVo;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.Location;
import com.mt.wms.core.dal.entity.VehicleInfo;
import com.mt.wms.core.dal.service.LocationServiceBiz;
import com.mt.wms.core.errorcode.ApiErrorCode;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/11/9 11:13
 * @Version 1.0
 */
@Service
@Transactional
public class LocationInfoServiceImpl extends BaseService implements LocationInfoService {

    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Override
    public R<LocationInfoVo> get(IdParam idParam) {
        Assert.notNull(ApiErrorCode.INVALID_PARAMETER,idParam.getId());
        Location location = locationServiceBiz.getById(idParam.getId());
        LocationInfoVo locationInfoVo=LocationInfoVo.builder().build();
        BeanUtils.copyProperties(location,locationInfoVo);
        return successful(locationInfoVo);
    }

    @Override
    public R<List<LocationInfoVo>> list(LocationInfoQueryParam locationInfoQueryParam) {
        QueryWrapper<Location> wrapper=new QueryWrapper<>();
        wrapper.eq(Location.VALID,1);
        List<Location> locationList = locationServiceBiz.list(wrapper);
        List<LocationInfoVo> locationInfoVoList = com.mt.wms.core.utils.BeanUtils.copyList(locationList, LocationInfoVo.class);
        return successful(locationInfoVoList);
    }

    @Override
    public R<PageVo<LocationInfoVo>> page(LocationInfoQueryParam locationInfoQueryParam) {
        QueryWrapper<Location> wrapper=new QueryWrapper<>();
        wrapper.eq(Location.VALID,1);
        Page<Location> page = locationServiceBiz.page(new Page<>(locationInfoQueryParam.getCurrent(), locationInfoQueryParam.getSize()), wrapper);
        return successful(new PageVo<>(page,LocationInfoVo.class));
    }

    @Override
    public R<IdVo> add(LocationInfoParam locationInfoParam) {
        QueryWrapper<Location> wrapper=new QueryWrapper<>();

        Location location=new Location();
        BeanUtils.copyProperties(locationInfoParam,location);
        setCommonField(location);
        locationServiceBiz.save(location);
        return successful(IdVo.builder().id(location.getId()).build());
    }

    @Override
    public R<IdVo> update(LocationInfoParam locationInfoParam) {
        Location location = locationServiceBiz.getById(locationInfoParam.getId());
        if (!locationInfoParam.getCode().equals(location.getCode())){
            QueryWrapper<Location> wrapper=new QueryWrapper<>();
            wrapper.eq(Location.CODE,locationInfoParam.getCode());
            wrapper.eq(Location.VALID,1);
            Assert.eqZero(locationServiceBiz.count(wrapper),"库位编码已存在！");
        }
        Location updateLocation=new Location();
        BeanUtils.copyProperties(locationInfoParam,updateLocation);
        setUpdateCommonField(updateLocation);
        locationServiceBiz.updateById(updateLocation);
        return successful(IdVo.builder().id(updateLocation.getId()).build());
    }

    @Override
    public R<IdVo> delete(IdParam idParam) {
        locationServiceBiz.removeById(idParam.getId());
        return successful(IdVo.builder().id(idParam.getId()).build());
    }
}
