package com.mt.wms.basic.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.basic.params.VehicleParam;
import com.mt.wms.basic.params.VehicleQueryParam;
import com.mt.wms.basic.service.VehicleService;
import com.mt.wms.basic.vo.AlarmBaseVo;
import com.mt.wms.basic.vo.VehicleVo;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.AlarmBase;
import com.mt.wms.core.dal.entity.VehicleInfo;
import com.mt.wms.core.dal.service.VehicleInfoServiceBiz;
import com.mt.wms.core.errorcode.ApiErrorCode;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/11/9 10:09
 * @Version 1.0
 */
@Service
@Transactional
public class VehicleServiceImpl extends BaseService implements VehicleService {

    @Resource
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @Override
    public R<VehicleVo> get(IdParam idParam) {
        Assert.notNull(ApiErrorCode.INVALID_PARAMETER,idParam.getId());
        VehicleInfo vehicleInfo = vehicleInfoServiceBiz.getById(idParam.getId());
        VehicleVo vehicleVo=VehicleVo.builder().build();
        BeanUtils.copyProperties(vehicleInfo,vehicleVo);
        return successful(vehicleVo);
    }

    @Override
    public R<List<VehicleVo>> list(VehicleQueryParam vehicleQueryParam) {
        QueryWrapper<VehicleInfo> wrapper=new QueryWrapper<>();
        wrapper.eq(VehicleInfo.VALID,1);
        List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(wrapper);
        List<VehicleVo> vehicleVoList = com.mt.wms.core.utils.BeanUtils.copyList(vehicleInfoList, VehicleVo.class);
        return successful(vehicleVoList);
    }

    @Override
    public R<PageVo<VehicleVo>> page(VehicleQueryParam vehicleQueryParam) {
        QueryWrapper<VehicleInfo> wrapper=new QueryWrapper<>();
        wrapper.eq(VehicleInfo.VALID,1);
        Page<VehicleInfo> page = vehicleInfoServiceBiz.page(new Page<>(vehicleQueryParam.getCurrent(), vehicleQueryParam.getSize()), wrapper);
        return successful(new PageVo<>(page,VehicleVo.class));
    }

    @Override
    public R<IdVo> add(VehicleParam vehicleParam) {
        QueryWrapper<VehicleInfo> wrapper=new QueryWrapper<>();

        VehicleInfo vehicleInfo=new VehicleInfo();
        BeanUtils.copyProperties(vehicleParam,vehicleInfo);
        setCommonField(vehicleInfo);
        vehicleInfoServiceBiz.save(vehicleInfo);
        return successful(IdVo.builder().id(vehicleInfo.getId()).build());
    }

    @Override
    public R<IdVo> update(VehicleParam vehicleParam) {
        VehicleInfo vehicleInfo = vehicleInfoServiceBiz.getById(vehicleParam.getId());
        if (!vehicleParam.getCode().equals(vehicleInfo.getCode())){
            QueryWrapper<VehicleInfo> wrapper=new QueryWrapper<>();
            wrapper.eq(VehicleInfo.CODE,vehicleParam.getCode());
            wrapper.eq(VehicleInfo.VALID,1);
            Assert.eqZero(vehicleInfoServiceBiz.count(wrapper),"车辆编码已存在！");
        }
        VehicleInfo updateVehicleInfo=new VehicleInfo();
        BeanUtils.copyProperties(vehicleParam,updateVehicleInfo);
        setUpdateCommonField(updateVehicleInfo);
        vehicleInfoServiceBiz.updateById(updateVehicleInfo);
        return successful(IdVo.builder().id(updateVehicleInfo.getId()).build());
    }

    @Override
    public R<IdVo> delete(IdParam idParam) {
        vehicleInfoServiceBiz.removeById(idParam.getId());
        return successful(IdVo.builder().id(idParam.getId()).build());
    }
}
