package com.mt.wms.empty.controller;

import cn.hutool.http.ContentType;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSON;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.CommunicationLog;
import com.mt.wms.core.dal.service.CommunicationLogServiceBiz;
import com.mt.wms.core.enums.WhetherEnum;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.*;
import com.mt.wms.empty.service.OrderInfoService;
import com.mt.wms.empty.vo.ApmsCreateProcessSheetVo;
import com.mt.wms.empty.vo.ApmsEndProcessVo;
import com.mt.wms.empty.vo.ApmsFinishProcessSheetVo;
import com.mt.wms.empty.vo.ApmsStoveVo;
import com.mt.wms.empty.websocket.WebSocketServer;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;

/**
 * @author xcc
 * @date 2021年11月18日
 * @since 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "apms")
@Slf4j
@Api(value = "Apms相关接口", tags = "Apms相关接口", hidden = false)
public class ApmsController extends BaseController {
    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private CommunicationLogServiceBiz communicationLogServiceBiz;
    @Autowired
    private WebSocketServer webSocketServer;

    static String Token = "";
    static LocalDateTime TokenExpireTime = LocalDateTime.now();
    static String ApiAddress = "http://192.168.1.109:80";

    @PostMapping(value = "apmsPostOrder")
    @ApiOperation(value = "WMS接收APMS推送订单信息")
    public R<String> apmsPostOrder(@Validated @RequestBody OrderParamForApms apmsPostOrderParam) {
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS接收APMS推送订单信息");
        communicationLog.setType(3);
        setCommonField(communicationLog);
        R<String> stringR = orderInfoService.apmsPostOrder(apmsPostOrderParam);
        communicationLog.setContent("param:" + apmsPostOrderParam.toString() + "\nresult:" + stringR.toString());
        communicationLogServiceBiz.save(communicationLog);
        return stringR;
    }

    @PostMapping(value = "apmsVoidOrder")
    @ApiOperation(value = "WMS接收APMS作废订单信息")
    public R<String> apmsVoidOrder(@Validated @RequestBody Iden idenCardNum) {
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS接收APMS作废订单信息");
        communicationLog.setType(3);
        setCommonField(communicationLog);
        R<String> stringR = orderInfoService.apmsVoidOrder(idenCardNum.getIdenCardNum());
        communicationLog.setContent("param:" + idenCardNum + "\nresult:" + stringR.toString());
        communicationLogServiceBiz.save(communicationLog);
        return stringR;
    }

    @PostMapping(value = "apmsCompleteOrder")
    @ApiOperation(value = "WMS接收APMS加工单检验结果")
    public R<String> apmsCompleteOrder(@Validated @RequestBody ApmsCompleteOrder apmsCompleteOrder) {
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS接收APMS加工单检验结果");
        communicationLog.setType(3);
        setCommonField(communicationLog);
        //APMS完成订单,返回检验结果，更新结果和后续处理
        R<String> stringR = orderInfoService.apmsCompleteOrder(apmsCompleteOrder);
        communicationLog.setContent("param:" + apmsCompleteOrder + "\nresult:" + stringR.toString());
        communicationLogServiceBiz.save(communicationLog);
        return stringR;
    }


    /**
     * 通过生产单编号删除生产单,如果生产单加工完成,不允许删除
     *
     * @param sheetNo 生产单编号
     * @return 删除结果
     */
    @PostMapping(value = "deleteBySheetNo")
    @ApiOperation(value = "通知APMS删除生产单", hidden = false)
    public R<ApmsEndProcessVo> deleteBySheetNo(@Validated @RequestBody String sheetNo) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("sheetNo", sheetNo);
        HttpResponse response = HttpUtil.createGet(ApiAddress + "/platform/api/deleteBySheetNo")
                .header("token", Token).form(paramMap).execute();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS删除生产单");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(JSONUtil.toBean(response.body(), ApmsEndProcessVo.class));
    }

    @PostMapping(value = "createProcessSheet")
    @ApiOperation(value = "通知APMS创建生产单接口", hidden = false)
    public R<ApmsCreateProcessSheetVo> createProcessSheet(@Validated @RequestBody ApmsCreateProcessSheet apmsCreateProcess) {
        checkToken();
        String paramJson = JSONUtil.toJsonStr(apmsCreateProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/createProcessSheet")
                .header("token", Token).body(paramJson, ContentType.JSON.toString()).execute();
        int status = response.getStatus();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS创建生产单");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramJson + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        String body = response.body();
        JSONObject jsonObject = com.alibaba.fastjson.JSON.parseObject(body);
        Object success = jsonObject.get("success");
        if (status == 200)
        {
            ApmsCreateProcessSheetVo apmsCreateProcessSheetVo = JSONUtil.toBean(response.body(), ApmsCreateProcessSheetVo.class);
            if (!apmsCreateProcessSheetVo.getSuccess()){
                try {
                    webSocketServer.sendtoAll(apmsCreateProcessSheetVo.getMsg());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return successful(apmsCreateProcessSheetVo);
        }
        else
        {
            return failed(status, response.body());
        }
    }

    @PostMapping(value = "startProcess")
    @ApiOperation(value = "通知APMS生产单开始处理", hidden = false)
    public R<ApmsStartProcess> startProcess(@Validated @RequestBody ApmsStartProcess apmsStartProcess) {
        checkToken();
        String paramJson = JSONUtil.toJsonStr(apmsStartProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/startProcess")
                .header("token", Token).body(paramJson, ContentType.JSON.toString()).execute();
        int status = response.getStatus();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS生产单开始处理");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramJson + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        if (status == 200)
        {
            return successful(JSONUtil.toBean(response.body(), ApmsStartProcess.class));
        }
        else
        {
            return failed(status, response.body());
        }
    }

    @PostMapping(value = "endProcess")
    @ApiOperation(value = "通知APMS生产单结束处理", hidden = false)
    public R<ApmsEndProcessVo> endProcess(@Validated @RequestBody ApmsEndProcess apmsEndProcess) {
        checkToken();
        String paramJson = JSONUtil.toJsonStr(apmsEndProcess);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/endProcess")
                .header("token", Token).body(paramJson, ContentType.JSON.toString()).execute();
        int status = response.getStatus();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS生产单结束处理");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramJson + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        if (status == 200)
        {
            return successful(JSONUtil.toBean(response.body(), ApmsEndProcessVo.class));
        }
        else
        {
            return failed(status, response.body());
        }
    }

    @PostMapping(value = "finishProcessSheet")
    @ApiOperation(value = "通知APMS完成生产单接口", hidden = false)
    public R<ApmsFinishProcessSheetVo> finishProcessSheet(@Validated @RequestBody ApmsFinishProcessSheet apmsFinishProcessSheet) {
        checkToken();
        String paramJson = JSONUtil.toJsonStr(apmsFinishProcessSheet);
        HttpResponse response = HttpUtil.createPost(ApiAddress + "/platform/api/finishProcessSheet")
                .header("token", Token).body(paramJson, ContentType.JSON.toString()).execute();
        int status = response.getStatus();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通知APMS完成生产单");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramJson + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        if (status == 200)
        {
            return successful(JSONUtil.toBean(response.body(), ApmsFinishProcessSheetVo.class));
        }
        else
        {
            return failed(status, response.body());
        }
    }

    @PostMapping(value = "getApmsToken")
    @ApiOperation(value = "获取APMS Token", hidden = true)
    public R<String> getApmsToken() {
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("apiKey", "7ee8f59bbaeae27e");
        paramMap.put("platFormCode", "CUC");
        String result = HttpUtil.get(ApiAddress + "/platform/api/getToken", paramMap);
        JSON parse = JSONUtil.parse(result);
        Object token = parse.getByPath("token");
        String expireTime = parse.getByPath("expireTime").toString();
        Token = token.toString();
        TokenExpireTime = LocalDateTime.parse(expireTime, DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS获取APMS Token");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + result);
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        return successful(result);
    }

    @PostMapping(value = "getStoveCodeByWorkShopCode")
    @ApiOperation(value = "WMS通过APMS使用车间编码获取炉号信息", hidden = false)
    public R<ApmsStoveVo> getStoveCodeByWorkShopCode(String workShopCode) {
        checkToken();
        HashMap<String, Object> paramMap = new HashMap<>();
        paramMap.put("workShopCode", workShopCode);
        HttpResponse response = HttpUtil.createGet(ApiAddress + "/platform/api/getStoveCodeByWorkShopCode")
                .header("token", Token).form(paramMap).execute();
        int status = response.getStatus();
        CommunicationLog communicationLog = new CommunicationLog();
        communicationLog.setCode("APMS" + System.currentTimeMillis());
        communicationLog.setLogName("WMS通过APMS使用车间编码获取炉号信息");
        communicationLog.setType(2);
        communicationLog.setContent("param:" + paramMap + "\nresult:" + response.body());
        setCommonField(communicationLog);
        communicationLogServiceBiz.save(communicationLog);
        if (status == 200)
        {
            return successful(JSONUtil.toBean(response.body(), ApmsStoveVo.class));
        }
        else
        {
            return failed(status, response.body());
        }
    }

    private void checkToken() {
        if (Token.isEmpty())
        {
            getApmsToken();
        }
        else if (TokenExpireTime.isBefore(LocalDateTime.now()))
        {
            getApmsToken();
        }
    }

    private void setCommonField(CommunicationLog communicationLog) {
        communicationLog.setValid(WhetherEnum.YES.getValue());
        communicationLog.setCreateTime(LocalDateTime.now());
        communicationLog.setUpdateTime(LocalDateTime.now());
    }
}
