package com.mt.wms.empty.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.gson.JsonObject;
import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.utils.BeanUtils;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.ElectricQueryParam;
import com.mt.wms.empty.vo.ApmsCheckResultVo;
import com.mt.wms.empty.vo.ConsumeDataVo;
import com.mt.wms.empty.vo.EnergyVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.validation.groups.Default;
import java.math.BigDecimal;
import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.*;

/**
 * @Author: liguanghao
 * @Date: 2021/12/16 11:20
 * @Version 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "electric")
@Slf4j
@Api(value = "电能消耗相关接口", tags = "电能消耗相关接口", hidden = false)
public class EnergyController extends BaseController {

    @Autowired
    private ParEleValueServiceBiz parEleValueServiceBiz;
    @Autowired
    private ParGasValueServiceBiz parGasValueServiceBiz;
    @Autowired
    private ParRotSpeedValueServiceBiz parRotSpeedValueServiceBiz;
    @Autowired
    private ParTemValueServiceBiz parTemValueServiceBiz;
    @Autowired
    private KilnInfoServiceBiz kilnInfoServiceBiz;
    @Autowired
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Autowired
    private TaskHisServiceBiz taskHisServiceBiz;
    @Autowired
    private ApmsCheckResultServiceBiz apmsCheckResultServiceBiz;


    @PostMapping(value = "energyList")
    @ApiOperation(value = "根据加工炉id获取能源消耗情况")
    public R<String> energyList(@Validated({Default.class}) @RequestBody ElectricQueryParam electricQueryParam) {
        List<Long> taskIds = electricQueryParam.getTaskIds();
        JSONObject json = new JSONObject();
        for (Long taskId : taskIds
        ) {
            //TaskHis currTask = taskHisServiceBiz.getById(electricQueryParam.getTaskId());
            TaskHis currTask = taskHisServiceBiz.getOne(new QueryWrapper<TaskHis>().eq(TaskHis.TASK_ID, taskId));
            Long kilnId = currTask.getKilnId();
            KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
            Integer kilnType = kilnInfo.getType();
            //传入任务号，工业炉号，查询消耗数据，返回json字符串
            JSONObject jsonObject = new JSONObject();
            ConsumeDataVo consumeDataVo = ConsumeDataVo.builder().build();

            Map<String, Object> map = new HashMap<>();
            //电能消耗
            List<ParEleValue> parEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parEleValueList.size() > 0) {
                List<Object> eleList = new ArrayList<>();
                //电能消耗
                List<Object> eleConsume = new ArrayList<>();
                //A向电压
                List<Object> aVolConsume = new ArrayList<>();
                //B向电压
                List<Object> bVolConsume = new ArrayList<>();
                //C向电压
                List<Object> cVolConsume = new ArrayList<>();
                Float talBat = 0F;
                int i = 0;
                for (ParEleValue parEleValue : parEleValueList
                ) {
                    //首位置为0
                    if (i == 0) {
                        talBat = parEleValue.getTotalBat();
                    }

                    i++;
                    map.put("time", parEleValue.getCreateTime());
                    map.put("value", floatSubtract(parEleValue.getTotalBat(), talBat));
                    map.put("name", "耗电量");
                    map.put("unit", "KW");
                    map.put("炉号", currTask.getSheetNo());
                    eleConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time", parEleValue.getCreateTime());
                    map.put("value", parEleValue.getAVoltagevValue());
                    map.put("name", "A向电压");
                    map.put("unit", "V");
                    map.put("炉号", currTask.getSheetNo());
                    aVolConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time", parEleValue.getCreateTime());
                    map.put("value", parEleValue.getBVoltagevValue());
                    map.put("name", "B向电压");
                    map.put("unit", "V");
                    map.put("炉号", currTask.getSheetNo());
                    bVolConsume.add(JSONObject.toJSON(map));
                    map.clear();
                    map.put("time", parEleValue.getCreateTime());
                    map.put("value", parEleValue.getCVoltagevValue());
                    map.put("name", "C向电压");
                    map.put("unit", "V");
                    map.put("炉号", currTask.getSheetNo());
                    cVolConsume.add(JSONObject.toJSON(map));
                    map.clear();
                }
                eleList.add(eleConsume);
                eleList.add(aVolConsume);
                eleList.add(bVolConsume);
                eleList.add(cVolConsume);
                jsonObject.put("ele", eleList);
            }
            //氮气、甲醇、丙烷、氨气消耗量
            List<ParGasValue> parGasValueList = parGasValueServiceBiz.list(new QueryWrapper<ParGasValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parGasValueList.size() > 0) {
                if (kilnType == 1) {
                    List<Object> gasList = new ArrayList<>();
                    //氮气
                    List<Object> nitConsume = new ArrayList<>();
                    Float nit = 0F;
                    //甲醇
                    List<Object> metConsume = new ArrayList<>();
                    Float met = 0F;
                    //丙烷
                    List<Object> propaneConsume = new ArrayList<>();
                    Float propane = 0F;
                    //氨气
                    List<Object> ammoniaConsume = new ArrayList<>();
                    Float ammonia = 0F;
                    int i = 0;
                    for (ParGasValue parGasValue : parGasValueList
                    ) {
                        //累计值的量首位置为0
                        if (i == 0) {
                            nit = parGasValue.getNitFlowValue();
                            met = parGasValue.getMethanolFlow();
                            propane = parGasValue.getPropaneFlow();
                            ammonia = parGasValue.getAmmoniaFlowValue();
                        }
                        i++;
                        map.put("time", parGasValue.getCreateTime());
                        map.put("value", floatSubtract(parGasValue.getNitFlowValue(), nit));
                        map.put("name", "氮气");
                        map.put("unit", "m³");
                        map.put("炉号", currTask.getSheetNo());
                        nitConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parGasValue.getCreateTime());
                        map.put("value", floatSubtract(parGasValue.getMethanolFlow(), met));
                        map.put("name", "甲醇");
                        map.put("unit", "m³");
                        map.put("炉号", currTask.getSheetNo());
                        metConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parGasValue.getCreateTime());
                        map.put("value", floatSubtract(parGasValue.getPropaneFlow(), propane));
                        map.put("name", "丙烷");
                        map.put("unit", "m³");
                        map.put("炉号", currTask.getSheetNo());
                        propaneConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parGasValue.getCreateTime());
                        map.put("value", floatSubtract(parGasValue.getAmmoniaFlowValue(), ammonia));
                        map.put("name", "氨气");
                        map.put("unit", "m³");
                        map.put("炉号", currTask.getSheetNo());
                        ammoniaConsume.add(JSONObject.toJSON(map));
                        map.clear();
                    }
                    gasList.add(nitConsume);
                    gasList.add(metConsume);
                    gasList.add(propaneConsume);
                    gasList.add(ammoniaConsume);
                    jsonObject.put("gas", gasList);
                }
                if (kilnType == 3) {
                    List<Object> gasList = new ArrayList<>();
                    //氮气
                    List<Object> nitConsume = new ArrayList<>();
                    Float nit = 0F;
                    //二氧化碳
                    List<Object> carbonConsume = new ArrayList<>();
                    Float carbon = 0F;
                    //氨气
                    List<Object> ammoniaConsume = new ArrayList<>();
                    Float ammonia = 0F;
                    int i = 0;
                    for (ParGasValue parGasValue : parGasValueList
                    ) {
                        //累计值的量首位置为0
                        if (i == 0) {
                            nit = parGasValue.getNitFlowValue();
                            carbon = parGasValue.getCarDioxideFlowValue();
                            ammonia = parGasValue.getAmmoniaFlowValue();
                        }
                        i++;
                        map.put("time", parGasValue.getCreateTime());
                        map.put("value", floatSubtract(parGasValue.getNitFlowValue(), nit));
                        map.put("name", "氮气");
                        map.put("unit", "m³");
                        map.put("炉号", currTask.getSheetNo());
                        nitConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parGasValue.getCreateTime());
                        map.put("value", floatSubtract(parGasValue.getCarDioxideFlowValue(), carbon));
                        map.put("name", "二氧化碳");
                        map.put("unit", "m³");
                        map.put("炉号", currTask.getSheetNo());
                        carbonConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parGasValue.getCreateTime());
                        map.put("value", floatSubtract(parGasValue.getAmmoniaFlowValue(), ammonia));
                        map.put("name", "氨气");
                        map.put("unit", "m³");
                        map.put("炉号", currTask.getSheetNo());
                        ammoniaConsume.add(JSONObject.toJSON(map));
                        map.clear();
                    }
                    gasList.add(nitConsume);
                    gasList.add(carbonConsume);
                    gasList.add(ammoniaConsume);
                    jsonObject.put("gas", gasList);
                }
            }
            //油搅拌转速趋势
            List<ParRotSpeedValue> parRotSpeedValueList = parRotSpeedValueServiceBiz.list(new QueryWrapper<ParRotSpeedValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parRotSpeedValueList.size() > 0) {
                if (kilnType == 1) {
                    List<Object> rotList = new ArrayList<>();
                    List<Object> rot1Consume = new ArrayList<>();
                    List<Object> rot2Consume = new ArrayList<>();
                    for (ParRotSpeedValue parRotSpeedValue : parRotSpeedValueList
                    ) {
                        map.put("time", parRotSpeedValue.getCreateTime());
                        map.put("value", parRotSpeedValue.getOilStiSpeedAValue());
                        map.put("name", "1号油搅拌转速");
                        map.put("unit", "转/min");
                        map.put("炉号", currTask.getSheetNo());
                        rot1Consume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parRotSpeedValue.getCreateTime());
                        map.put("value", parRotSpeedValue.getOilStiSpeedBValue());
                        map.put("name", "2号油搅拌转速");
                        map.put("unit", "转/min");
                        map.put("炉号", currTask.getSheetNo());
                        rot2Consume.add(JSONObject.toJSON(map));
                        map.clear();
                    }
                    rotList.add(rot1Consume);
                    rotList.add(rot2Consume);
                    jsonObject.put("rot", rotList);
                }
            }
            //温度趋势（实际温度、设定温度）
            List<ParTemValue> parTemValueList = parTemValueServiceBiz.list(new QueryWrapper<ParTemValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parTemValueList.size() > 0) {
                if (kilnType == 1) {
                    List<Object> tempList = new ArrayList<>();
                    List<Object> actualTempConsume = new ArrayList<>();
                    List<Object> setupTempConsume = new ArrayList<>();
                    List<Object> actualOilTempConsume = new ArrayList<>();
                    List<Object> setupOilTempConsume = new ArrayList<>();
                    for (ParTemValue parTemValue : parTemValueList
                    ) {
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getActTemValue());
                        map.put("name", "实际温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        actualTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getSetTemValue());
                        map.put("name", "设定温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        setupTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getOilTankActTemValue());
                        map.put("name", "油槽实际温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        actualOilTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getOilTankSetTemValue());
                        map.put("name", "油槽设定温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        setupOilTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                    }
                    tempList.add(actualTempConsume);
                    tempList.add(setupTempConsume);
                    tempList.add(actualOilTempConsume);
                    tempList.add(setupOilTempConsume);
                    jsonObject.put("temp", tempList);
                }
                if (kilnType == 2) {
                    List<Object> tempList = new ArrayList<>();
                    List<Object> actualTempConsume = new ArrayList<>();
                    List<Object> setupTempConsume = new ArrayList<>();
                    for (ParTemValue parTemValue : parTemValueList
                    ) {
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getActTemValue());
                        map.put("name", "实际温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        actualTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getSetTemValue());
                        map.put("name", "设定温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        setupTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                    }
                    tempList.add(actualTempConsume);
                    tempList.add(setupTempConsume);
                    jsonObject.put("temp", tempList);
                }
                if (kilnType == 3) {

                    List<Object> tempList = new ArrayList<>();
                    List<Object> actualTempConsume = new ArrayList<>();
                    List<Object> setupTempConsume = new ArrayList<>();
                    List<Object> outerZone1TempConsume = new ArrayList<>();
                    List<Object> outerZone2TempConsume = new ArrayList<>();
                    for (ParTemValue parTemValue : parTemValueList
                    ) {
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getActTemValue());
                        map.put("name", "实际温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        actualTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getSetTemValue());
                        map.put("name", "设定温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        setupTempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parTemValue.getCreateTime());
                        map.put("value", parTemValue.getOuterZone1Temp());
                        map.put("name", "外一区温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        outerZone1TempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                        map.put("time", parTemValue.getCreateTime());

                        map.put("value", parTemValue.getOuterZone2Temp());
                        map.put("name", "外二区温度");
                        map.put("unit", "℃");
                        map.put("炉号", currTask.getSheetNo());
                        outerZone2TempConsume.add(JSONObject.toJSON(map));
                        map.clear();
                    }
                    tempList.add(actualTempConsume);
                    tempList.add(setupTempConsume);
                    tempList.add(outerZone1TempConsume);
                    tempList.add(outerZone2TempConsume);
                    jsonObject.put("temp", tempList);
                }
            }
            json.put(String.valueOf(taskId), jsonObject);
        }
        return R.ok(json.toJSONString());
    }

    private Float floatSubtract(Float a, Float b) {
        BigDecimal a1 = new BigDecimal(Float.toString(a));
        BigDecimal b1 = new BigDecimal(Float.toString(b));
        return a1.subtract(b1).floatValue();
    }

    @PostMapping(value = "checkResultList")
    @ApiOperation(value = "根据生产单号获取apms检验结果")
    public List<ApmsCheckResult> getApmsCheckResult(@RequestBody ElectricQueryParam electricQueryParam) {
        return apmsCheckResultServiceBiz.list(new QueryWrapper<ApmsCheckResult>().eq(ApmsCheckResult.SHEET_NO, electricQueryParam.getSheetNo()));
    }

    @PostMapping(value = "eletric")
    @ApiOperation(value = "根据任务id获取消耗情况", hidden = false)
    public R<JSONObject> list(@Validated({Default.class}) @RequestBody ElectricQueryParam electricQueryParam) {
        List<Long> taskIds = electricQueryParam.getTaskIds();
        Map<String, Object> map = new HashMap<>();
        //电度值
        List<List<EnergyVo>> eleConsumeList = new ArrayList<>();
        //A向电压
        List<List<EnergyVo>> aVolConsumeList = new ArrayList<>();
        //B向电压
        List<List<EnergyVo>> bVolConsumeList = new ArrayList<>();
        //C向电压
        List<List<EnergyVo>> cVolConsumeList = new ArrayList<>();
        //A向电流
        List<List<EnergyVo>> aCurrentList = new ArrayList<>();
        //B向电流
        List<List<EnergyVo>> bCurrentList = new ArrayList<>();
        //C向电流
        List<List<EnergyVo>> cCurrentList = new ArrayList<>();
        //氮气
        List<List<EnergyVo>> nitConsumeList = new ArrayList<>();
        //甲醇
        List<List<EnergyVo>> metConsumeList = new ArrayList<>();
        //丙烷
        List<List<EnergyVo>> propaneConsumeList = new ArrayList<>();
        //氨气
        List<List<EnergyVo>> ammoniaConsumeList = new ArrayList<>();
        //二氧化碳
        List<List<EnergyVo>> carbonConsumeList = new ArrayList<>();
        //1号油搅拌转速
        List<List<EnergyVo>> rot1ConsumeList = new ArrayList<>();
        //2号油搅拌转速
        List<List<EnergyVo>> rot2ConsumeList = new ArrayList<>();
        //温度
        List<List<EnergyVo>> tempConsumeList = new ArrayList<>();
        //油槽温度
        List<List<EnergyVo>> oilTempConsumeList = new ArrayList<>();
        //外一区温度
        List<List<EnergyVo>> outerZone1TempConsumeList = new ArrayList<>();
        //外二区温度
        List<List<EnergyVo>> outerZone2TempConsumeList = new ArrayList<>();
        //碳/氮势
        List<List<EnergyVo>> carbonList = new ArrayList<>();
        List<List<ApmsCheckResultVo>> checkResultList = new ArrayList<>();
        for (Long taskId : taskIds
        ) {
            TaskHis currTask = taskHisServiceBiz.getOne(new QueryWrapper<TaskHis>().eq(TaskHis.TASK_ID, taskId));
            //防止早期没有炉号的报错
            if (currTask.getStoveCode() == null) {
                currTask.setStoveCode(currTask.getSheetNo());
            }
            //获取检验结果
            List<ApmsCheckResult> apmsCheckResults = apmsCheckResultServiceBiz.list(new QueryWrapper<ApmsCheckResult>()
                    .eq(ApmsCheckResult.SHEET_NO, currTask.getSheetNo()));
            if (apmsCheckResults.size() != 0) {
                List<ApmsCheckResultVo> apmsCheckResultVos = BeanUtils.copyList(apmsCheckResults, ApmsCheckResultVo.class);
                for (ApmsCheckResultVo apmsCheckResultVo : apmsCheckResultVos
                ) {
                    apmsCheckResultVo.setStoveCode(currTask.getStoveCode());
                }
                checkResultList.add(apmsCheckResultVos);
            }

            Long kilnId = currTask.getKilnId();
            KilnInfo kilnInfo = kilnInfoServiceBiz.getById(kilnId);
            Integer kilnType = kilnInfo.getType();
            //电能消耗
            List<ParEleValue> parEleValueList = parEleValueServiceBiz.list(new QueryWrapper<ParEleValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parEleValueList.size() > 0) {
                List<Object> eleList = new ArrayList<>();
                //电能消耗
                List<EnergyVo> eleConsume = new ArrayList<>();
                //A向电压
                List<EnergyVo> aVolConsume = new ArrayList<>();
                //B向电压
                List<EnergyVo> bVolConsume = new ArrayList<>();
                //C向电压
                List<EnergyVo> cVolConsume = new ArrayList<>();
                //A向电流
                List<EnergyVo> aCurrentConsume = new ArrayList<>();
                //B向电流
                List<EnergyVo> bCurrentConsume = new ArrayList<>();
                //C向电流
                List<EnergyVo> cCurrentConsume = new ArrayList<>();
                Float talBat = 0F;
                int i = 0;
                for (ParEleValue parEleValue : parEleValueList
                ) {
                    //首位置为0
                    if (i == 0) {
                        talBat = parEleValue.getTotalBat();
                    }
                    i++;
                    EnergyVo eleVo = EnergyVo.builder().build();
                    eleVo.setCode(currTask.getStoveCode());
                    eleVo.setName("耗电量");
                    eleVo.setUnit("KW");
                    eleVo.setTime(parEleValue.getCreateTime());
                    eleVo.setValue(floatSubtract(parEleValue.getTotalBat(), talBat));
                    eleConsume.add(eleVo);

                    EnergyVo aVolVo = EnergyVo.builder().build();
                    aVolVo.setCode(currTask.getStoveCode());
                    aVolVo.setName("A向电压");
                    aVolVo.setUnit("V");
                    aVolVo.setTime(parEleValue.getCreateTime());
                    aVolVo.setValue(parEleValue.getAVoltagevValue());
                    aVolConsume.add(aVolVo);

                    EnergyVo bVolVo = EnergyVo.builder().build();
                    bVolVo.setCode(currTask.getStoveCode());
                    bVolVo.setName("B向电压");
                    bVolVo.setUnit("V");
                    bVolVo.setTime(parEleValue.getCreateTime());
                    bVolVo.setValue(parEleValue.getBVoltagevValue());
                    bVolConsume.add(bVolVo);

                    EnergyVo cVolVo = EnergyVo.builder().build();
                    cVolVo.setCode(currTask.getStoveCode());
                    cVolVo.setName("C向电压");
                    cVolVo.setUnit("V");
                    cVolVo.setTime(parEleValue.getCreateTime());
                    cVolVo.setValue(parEleValue.getCVoltagevValue());
                    cVolConsume.add(cVolVo);

                    EnergyVo aCurrVo = EnergyVo.builder().build();
                    aCurrVo.setCode(currTask.getStoveCode());
                    aCurrVo.setName("A向电流");
                    aCurrVo.setUnit("A");
                    aCurrVo.setTime(parEleValue.getCreateTime());
                    aCurrVo.setValue(parEleValue.getACurrentValue());
                    aCurrentConsume.add(aCurrVo);

                    EnergyVo bCurrVo = EnergyVo.builder().build();
                    bCurrVo.setCode(currTask.getStoveCode());
                    bCurrVo.setName("B向电流");
                    bCurrVo.setUnit("A");
                    bCurrVo.setTime(parEleValue.getCreateTime());
                    bCurrVo.setValue(parEleValue.getBCurrentValue());
                    bCurrentConsume.add(bCurrVo);

                    EnergyVo cCurrVo = EnergyVo.builder().build();
                    cCurrVo.setCode(currTask.getStoveCode());
                    cCurrVo.setName("C向电流");
                    cCurrVo.setUnit("A");
                    cCurrVo.setTime(parEleValue.getCreateTime());
                    cCurrVo.setValue(parEleValue.getCCurrentValue());
                    cCurrentConsume.add(cCurrVo);
                }
                eleConsumeList.add(eleConsume);
                aVolConsumeList.add(aVolConsume);
                bVolConsumeList.add(bVolConsume);
                cVolConsumeList.add(cVolConsume);
                aCurrentList.add(aCurrentConsume);
                bCurrentList.add(bCurrentConsume);
                cCurrentList.add(cCurrentConsume);

            }
            //氮气、甲醇、丙烷、氨气二氧化碳消耗量碳势
            List<ParGasValue> parGasValueList = parGasValueServiceBiz.list(new QueryWrapper<ParGasValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parGasValueList.size() > 0) {
                if (kilnType == 1) {
                    //碳/氮势
                    List<EnergyVo> carbon = new ArrayList<>();
                    //碳/氮势-设定
                    List<EnergyVo> carbonSet = new ArrayList<>();
                    //氮气
                    List<EnergyVo> nitConsume = new ArrayList<>();
                    //甲醇
                    List<EnergyVo> metConsume = new ArrayList<>();
                    //丙烷
                    List<EnergyVo> propaneConsume = new ArrayList<>();
                    //氨气
                    List<EnergyVo> ammoniaConsume = new ArrayList<>();
                    for (ParGasValue parGasValue : parGasValueList
                    ) {
                        EnergyVo carbonSetVo = EnergyVo.builder().build();
                        carbonSetVo.setTime(parGasValue.getCreateTime());
                        carbonSetVo.setCode(currTask.getStoveCode().concat("-设定"));
                        carbonSetVo.setUnit("%");
                        carbonSetVo.setName("碳/氮势");
                        carbonSetVo.setValue(parGasValue.getSetNitPotValue());
                        carbonSet.add(carbonSetVo);
                        EnergyVo carbonActVo = EnergyVo.builder().build();
                        carbonActVo.setValue(parGasValue.getActualNitPotValue());
                        carbonActVo.setUnit("%");
                        carbonActVo.setName("碳/氮势");
                        carbonActVo.setCode(currTask.getStoveCode());
                        carbon.add(carbonActVo);

                        EnergyVo nitVo = EnergyVo.builder().build();
                        nitVo.setTime(parGasValue.getCreateTime());
                        nitVo.setUnit("m³/h");
                        nitVo.setValue(parGasValue.getNitFlowValue());
                        nitVo.setName("氮气");
                        nitVo.setCode(currTask.getStoveCode());
                        nitConsume.add(nitVo);

                        EnergyVo metVo = EnergyVo.builder().build();
                        metVo.setCode(currTask.getStoveCode());
                        metVo.setValue(parGasValue.getMethanolFlow());
                        metVo.setName("甲醇");
                        metVo.setUnit("m³/h");
                        metVo.setTime(parGasValue.getCreateTime());
                        metConsume.add(metVo);

                        EnergyVo propaneVo = EnergyVo.builder().build();
                        propaneVo.setTime(parGasValue.getCreateTime());
                        propaneVo.setCode(currTask.getStoveCode());
                        propaneVo.setUnit("m³/h");
                        propaneVo.setValue(parGasValue.getPropaneFlow());
                        propaneVo.setName("丙烷");
                        propaneConsume.add(propaneVo);

                        EnergyVo ammoniaVo = EnergyVo.builder().build();
                        ammoniaVo.setName("氨气");
                        ammoniaVo.setCode(currTask.getStoveCode());
                        ammoniaVo.setValue(parGasValue.getAmmoniaFlowValue());
                        ammoniaVo.setUnit("m³/h");
                        ammoniaVo.setTime(parGasValue.getCreateTime());
                        ammoniaConsume.add(ammoniaVo);
                    }
                    carbonList.add(carbon);
                    carbonList.add(carbonSet);
                    nitConsumeList.add(nitConsume);
                    metConsumeList.add(metConsume);
                    propaneConsumeList.add(propaneConsume);
                    ammoniaConsumeList.add(ammoniaConsume);
                }
                if (kilnType == 3) {
                    //碳/氮势
                    List<EnergyVo> carbon = new ArrayList<>();
                    //碳/氮势-设定
                    List<EnergyVo> carbonSet = new ArrayList<>();
                    //氮气
                    List<EnergyVo> nitConsume = new ArrayList<>();
                    //二氧化碳
                    List<EnergyVo> carbonConsume = new ArrayList<>();
                    //氨气
                    List<EnergyVo> ammoniaConsume = new ArrayList<>();
                    for (ParGasValue parGasValue : parGasValueList
                    ) {
                        EnergyVo carbonSetVo = EnergyVo.builder().build();
                        carbonSetVo.setTime(parGasValue.getCreateTime());
                        carbonSetVo.setCode(currTask.getStoveCode().concat("-设定"));
                        carbonSetVo.setUnit("%");
                        carbonSetVo.setName("碳/氮势");
                        carbonSetVo.setValue(parGasValue.getSetNitPotValue());
                        carbonSet.add(carbonSetVo);
                        EnergyVo carbonActVo = EnergyVo.builder().build();
                        carbonActVo.setValue(parGasValue.getActualNitPotValue());
                        carbonActVo.setUnit("%");
                        carbonActVo.setName("碳/氮势");
                        carbonActVo.setCode(currTask.getStoveCode());
                        carbon.add(carbonActVo);

                        EnergyVo nitVo = EnergyVo.builder().build();
                        nitVo.setTime(parGasValue.getCreateTime());
                        nitVo.setUnit("m³/h");
                        nitVo.setValue(parGasValue.getNitFlowValue());
                        nitVo.setName("氮气");
                        nitVo.setCode(currTask.getStoveCode());
                        nitConsume.add(nitVo);

                        EnergyVo carbonVo = EnergyVo.builder().build();
                        carbonVo.setTime(parGasValue.getCreateTime());
                        carbonVo.setCode(currTask.getStoveCode());
                        carbonVo.setUnit("m³/h");
                        carbonVo.setValue(parGasValue.getCarDioxideFlowValue());
                        carbonVo.setName("二氧化碳");
                        carbonConsume.add(carbonVo);

                        EnergyVo ammoniaVo = EnergyVo.builder().build();
                        ammoniaVo.setName("氨气");
                        ammoniaVo.setCode(currTask.getStoveCode());
                        ammoniaVo.setValue(parGasValue.getAmmoniaFlowValue());
                        ammoniaVo.setUnit("m³/h");
                        ammoniaVo.setTime(parGasValue.getCreateTime());
                        ammoniaConsume.add(ammoniaVo);
                    }
                    carbonList.add(carbon);
                    carbonList.add(carbonSet);
                    carbonConsumeList.add(carbonConsume);
                    nitConsumeList.add(nitConsume);
                    ammoniaConsumeList.add(ammoniaConsume);
                }
            }
            //油搅拌转速趋势
            List<ParRotSpeedValue> parRotSpeedValueList = parRotSpeedValueServiceBiz.list(new QueryWrapper<ParRotSpeedValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parRotSpeedValueList.size() > 0) {
                if (kilnType == 1) {
                    List<EnergyVo> rot1Consume = new ArrayList<>();
                    for (ParRotSpeedValue parRotSpeedValue : parRotSpeedValueList
                    ) {
                        EnergyVo rot1Vo = EnergyVo.builder().build();
                        rot1Vo.setCode(currTask.getStoveCode());
                        rot1Vo.setUnit("转/min");
                        rot1Vo.setName("1号油搅拌转速");
                        rot1Vo.setValue(parRotSpeedValue.getOilStiSpeedAValue());
                        rot1Vo.setTime(parRotSpeedValue.getCreateTime());
                        rot1Consume.add(rot1Vo);
                    }
                    rot1ConsumeList.add(rot1Consume);
                }
            }
            //温度趋势（实际温度、设定温度）油槽温度外区温度
            List<ParTemValue> parTemValueList = parTemValueServiceBiz.list(new QueryWrapper<ParTemValue>()
                    .eq(ParEleValue.KILN_ID, kilnId)
                    .eq(ParEleValue.TASK_ID, taskId));
            if (parTemValueList.size() > 0) {
                if (kilnType == 1) {
                    List<EnergyVo> actualTempConsume = new ArrayList<>();
                    List<EnergyVo> setupTempConsume = new ArrayList<>();
                    List<EnergyVo> actualOilTempConsume = new ArrayList<>();
                    List<EnergyVo> setupOilTempConsume = new ArrayList<>();
                    for (ParTemValue parTemValue : parTemValueList
                    ) {
                        EnergyVo actTempVo = EnergyVo.builder().build();
                        actTempVo.setTime(parTemValue.getCreateTime());
                        actTempVo.setCode(currTask.getStoveCode());
                        actTempVo.setUnit("℃");
                        actTempVo.setValue(parTemValue.getActTemValue());
                        actTempVo.setName("实际温度");
                        actualTempConsume.add(actTempVo);

                        EnergyVo setTempVo = EnergyVo.builder().build();
                        setTempVo.setTime(parTemValue.getCreateTime());
                        setTempVo.setCode(currTask.getStoveCode().concat("-设定"));
                        setTempVo.setUnit("℃");
                        setTempVo.setValue(parTemValue.getSetTemValue());
                        setTempVo.setName("设定温度");
                        setupTempConsume.add(setTempVo);

                        EnergyVo oilSetTempVo = EnergyVo.builder().build();
                        oilSetTempVo.setTime(parTemValue.getCreateTime());
                        oilSetTempVo.setCode(currTask.getStoveCode().concat("-设定"));
                        oilSetTempVo.setUnit("℃");
                        oilSetTempVo.setValue(parTemValue.getOilTankSetTemValue());
                        oilSetTempVo.setName("油槽设定温度");
                        setupOilTempConsume.add(oilSetTempVo);

                        EnergyVo oilActTempVo = EnergyVo.builder().build();
                        oilActTempVo.setTime(parTemValue.getCreateTime());
                        oilActTempVo.setCode(currTask.getStoveCode());
                        oilActTempVo.setUnit("℃");
                        oilActTempVo.setValue(parTemValue.getOilTankActTemValue());
                        oilActTempVo.setName("油槽实际温度");
                        actualOilTempConsume.add(oilActTempVo);

                    }
                    tempConsumeList.add(actualTempConsume);
                    tempConsumeList.add(setupTempConsume);
                    oilTempConsumeList.add(actualOilTempConsume);
                    oilTempConsumeList.add(setupOilTempConsume);
                }
                if (kilnType == 2) {
                    List<EnergyVo> actualTempConsume = new ArrayList<>();
                    List<EnergyVo> setupTempConsume = new ArrayList<>();
                    for (ParTemValue parTemValue : parTemValueList
                    ) {
                        EnergyVo actTempVo = EnergyVo.builder().build();
                        actTempVo.setTime(parTemValue.getCreateTime());
                        actTempVo.setCode(currTask.getStoveCode());
                        actTempVo.setUnit("℃");
                        actTempVo.setValue(parTemValue.getActTemValue());
                        actTempVo.setName("实际温度");
                        actualTempConsume.add(actTempVo);

                        EnergyVo setTempVo = EnergyVo.builder().build();
                        setTempVo.setTime(parTemValue.getCreateTime());
                        setTempVo.setCode(currTask.getStoveCode().concat("-设定"));
                        setTempVo.setUnit("℃");
                        setTempVo.setValue(parTemValue.getSetTemValue());
                        setTempVo.setName("设定温度");
                        setupTempConsume.add(setTempVo);
                    }
                    tempConsumeList.add(actualTempConsume);
                    tempConsumeList.add(setupTempConsume);
                    ;
                }
                if (kilnType == 3) {

                    List<EnergyVo> actualTempConsume = new ArrayList<>();
                    List<EnergyVo> setupTempConsume = new ArrayList<>();
                    List<EnergyVo> outerZone1TempConsume = new ArrayList<>();
                    List<EnergyVo> outerZone2TempConsume = new ArrayList<>();
                    for (ParTemValue parTemValue : parTemValueList
                    ) {
                        EnergyVo actTempVo = EnergyVo.builder().build();
                        actTempVo.setTime(parTemValue.getCreateTime());
                        actTempVo.setCode(currTask.getStoveCode());
                        actTempVo.setUnit("℃");
                        actTempVo.setValue(parTemValue.getActTemValue());
                        actTempVo.setName("实际温度");
                        actualTempConsume.add(actTempVo);

                        EnergyVo setTempVo = EnergyVo.builder().build();
                        setTempVo.setTime(parTemValue.getCreateTime());
                        setTempVo.setCode(currTask.getStoveCode().concat("-设定"));
                        setTempVo.setUnit("℃");
                        setTempVo.setValue(parTemValue.getSetTemValue());
                        setTempVo.setName("设定温度");
                        setupTempConsume.add(setTempVo);

                    }
                    tempConsumeList.add(actualTempConsume);
                    tempConsumeList.add(setupTempConsume);
                }
            }
        }
        map.put("checkResult", checkResultList);
        map.put("ele", eleConsumeList);
        map.put("aVol", aVolConsumeList);
        map.put("bVol", bVolConsumeList);
        map.put("cVol", cVolConsumeList);
        map.put("aCurr", aCurrentList);
        map.put("bCurr", bCurrentList);
        map.put("cCurr", cCurrentList);
        map.put("carbon", carbonList);
        map.put("nit", nitConsumeList);
        map.put("met", metConsumeList);
        map.put("propane", propaneConsumeList);
        map.put("ammon", ammoniaConsumeList);
        map.put("car", carbonConsumeList);
        map.put("rot1", rot1ConsumeList);
        map.put("temp", tempConsumeList);
        map.put("oilTemp", oilTempConsumeList);
        return successful(JSONObject.parseObject(JSON.toJSONString(map)));
    }
}
