package com.mt.wms.empty.service;

import com.mt.wms.core.dal.entity.CurrTaskDet;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.CurrTaskDetQueryParam;
import com.mt.wms.empty.params.OrderInfoTaskDetParam;
import com.mt.wms.empty.params.TaskCreateDetParam;
import com.mt.wms.empty.vo.CurrTaskDetVo;
import com.mt.wms.empty.vo.OrderInfoTaskDetVo;

import java.util.HashMap;
import java.util.List;

/**
 * 拆分任务相关服务
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
public interface CurrTaskDetService {
    /**
     * 获取当前订单的任务详情
     * @param param 订单号 和标识卡
     * @return 结果
     */
    PageVo<OrderInfoTaskDetVo> getCurrOrderCurrTask(OrderInfoTaskDetParam param);

    /**
     * 创建正常加工的详细任务信息
     *
     * @param taskDet 详细任务信息
     * @param taskId  主任务id
     */
    void createProcessTaskDet(List<TaskCreateDetParam> taskDet, Long taskId);

    /**
     * 查询正常加工的详细任务信息
     *
     * @param currTaskId 详细任务信息
     * @return 结果
     */
    R<PageVo<CurrTaskDetVo>> getTaskDetailsPage(CurrTaskDetQueryParam currTaskId);

    /**
     * 查询某个主任务下的详细任务列表
     *
     * @param idParam 主任务id
     * @return 结果
     */
    List<CurrTaskDet> getTaskDetailsByMainId(IdParam idParam);

    /**
     * 查询某个标识卡号的详细任务列表
     *
     * @param idenCardNum 标识卡号
     * @return 结果
     */
    List<CurrTaskDet> getTaskDetailsByIdenCardNum(String idenCardNum);

    /**
     * 通过生产单删除某个生产单
     *
     * @param currTaskId 生产单号
     * @return 结果
     */
    R<String> deleteByCurrTaskId(Long currTaskId);

    /**
     * 通过标识卡号查询订单统计数
     *
     * @param idCard 任务详情历史
     * @return 结果
     */
    HashMap<String, Double> getSumByIdCard(String idCard);
}
