package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.params.BasePageParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.utils.IDGenerator;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.controller.ApmsController;
import com.mt.wms.empty.enums.TaskTypeEnum;
import com.mt.wms.empty.params.*;
import com.mt.wms.empty.service.*;
import com.mt.wms.empty.task.RunTaskUtils;
import com.mt.wms.empty.task.StoveCodeUtils;
import com.mt.wms.empty.task.TaskDistanceUtils;
import com.mt.wms.empty.vo.*;
import com.mt.wms.empty.websocket.WebSocketServer;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.IOException;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class CurrTaskServiceImpl extends BaseService implements CurrTaskService {

    @Autowired
    CurrTaskServiceBiz currTaskServiceBiz;
    @Autowired
    CurrTaskDetService currTaskDetService;
    @Autowired
    TaskHisService taskHisService;
    @Autowired
    TaskDetHisService taskDetHisService;
    @Autowired
    ApmsController apmsControl;
    @Autowired
    WebSocketServer webSocketServer;
    @Autowired
    OrderInfoServiceBiz orderInfoServiceBiz;
    @Autowired
    CraftInfoServiceBiz craftInfoServiceBiz;
    @Autowired
    VehicleInfoServiceBiz vehicleInfoServiceBiz;
    @Autowired
    TaskDistanceUtils taskDistanceUtils;
    @Autowired
    RunTaskUtils runTaskUtils;
    @Autowired
    AutoExeTaskServiceBiz autoExeTaskServiceBiz;
    @Autowired
    KilnInfoServiceBiz kilnInfoServiceBiz;
    @Autowired
    LocationServiceBiz locationServiceBiz;
    @Autowired
    CurrTaskDetServiceBiz currTaskDetServiceBiz;
    @Autowired
    InStockInfoServiceBiz inStockInfoServiceBiz;
    private Logger logger = LoggerFactory.getLogger(getClass());

    @Override
    public CurrTaskVo getCurrTask(IdParam idParam) {
        CurrTask byId = currTaskServiceBiz.getById(idParam.getId());
        CurrTaskVo build = CurrTaskVo.builder().build();
        BeanUtils.copyProperties(byId, build);
        return build;
    }

    @Override
    public CurrTask getCurrTaskById(Long id) {
        return currTaskServiceBiz.getById(id);
    }

    @Override
    public void updateCurrTaskById(CurrTask id) {
        currTaskServiceBiz.updateById(id);
    }

    @Override
    public R<IdVo> createProcessTask(TaskCreateParam param) {
        Assert.notNull("错误，加工类型不可为空！", param.getProcessType());
        CurrTask currTask = new CurrTask();
        BeanUtils.copyProperties(param, currTask);
        currTask.setTaskType(TaskTypeEnum.WTK.getValue());
        KilnInfo kilnInfo = kilnInfoServiceBiz.getById(currTask.getKilnId());
        if (param.getAutoTaskType() != 0) {
            currTask.setIsAuto(1);
        }

        //根据工艺号查询工艺id以及工艺plc值
        CraftInfo one = craftInfoServiceBiz.getOne(new QueryWrapper<CraftInfo>().eq(CraftInfo.CRAFT_CODE, param.getCraftCode()));
        currTask.setTargetPosition(kilnInfo.getCode());
        currTask.setCraftCodeId(one.getId());
        currTask.setPlcValue(one.getPlcValue());
        currTask.setInterCode(IDGenerator.gen("RW", IDGenerator.PATTERN_YYYYMMDDHHMM, 2, "TASK_CODE"));
        currTask.setTaskCode(currTask.getInterCode());
        setCommonField(currTask);
        currTaskServiceBiz.save(currTask);
        return successful(IdVo.builder().id(currTask.getId()).build());
    }

    @Override
    public R<PageVo<CurrTaskQueryVo>> currentTaskPage(CurrTaskQueryParam param) {
        QueryWrapper<CurrTask> currTaskQueryWrapper = new QueryWrapper<>();
        currTaskQueryWrapper.eq(CurrTask.VALID, 1);
        currTaskQueryWrapper.in(CurrTask.STATUS, 0, 1);
        Page<CurrTask> page = currTaskServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), currTaskQueryWrapper);
        return successful(new PageVo<>(page, CurrTaskQueryVo.class));
    }

    @Override
    public R<List<CurrTaskInfoVo>> currentTaskInfoPage(CurrTaskQueryParam param) {
        QueryWrapper<CurrTask> wrapper = new QueryWrapper<>();
        wrapper.eq(CurrTask.VALID, 1);
        wrapper.in(CurrTask.STATUS, 0, 1);
        List<CurrTask> currTaskList = currTaskServiceBiz.list(wrapper);
        List<CurrTaskInfoVo> list = new ArrayList<>();
        for (CurrTask currTask : currTaskList
        ) {
            CurrTaskInfoVo currTaskInfoVo = CurrTaskInfoVo.builder().build();
            //是否多步骤任务
            currTaskInfoVo.setIsAuto(currTask.getIsAuto());
            //任务状态
            if (currTask.getStatus() == 1) {
                currTaskInfoVo.setStatus(currTask.getStatus());
            } else if (currTask.getIsCache() == 1) {
                currTaskInfoVo.setStatus(2);
            } else {
                currTaskInfoVo.setStatus(0);
            }
            //生产单号
            if (currTask.getSheetNo() != null) {
                currTaskInfoVo.setSheetNo(currTask.getSheetNo());
            }
            currTaskInfoVo.setCreateTime(currTask.getCreateTime());
            //开始位置
            currTaskInfoVo.setStartPosition(currTask.getStartPosition());
            //currTask
            if (currTask.getIsAuto() == 1) {
                List<CurrTask> currTasks = currTaskServiceBiz
                        .list(new QueryWrapper<CurrTask>()
                                .eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId())
                                .orderByAsc(CurrTask.ID));
                List<CurrTaskVo> currTaskVos = com.mt.wms.core.utils.BeanUtils.copyList(currTasks, CurrTaskVo.class);
                currTaskInfoVo.setCurrTaskVoList(currTaskVos);
                String allStep = "液压台";
                String nowStep = "未开始";
                for (CurrTask currTask1 : currTasks
                ) {
                    InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.TASK_ID, currTask1.getId()));
                    if (inStockInfo != null) {
                        List<InStockInfoVo> inStockInfoVoList = new ArrayList<>();
                        InStockInfoVo inStockInfoVo = InStockInfoVo.builder().build();
                        BeanUtils.copyProperties(inStockInfo, inStockInfoVo);
                        inStockInfoVoList.add(inStockInfoVo);
                        currTaskInfoVo.setInStockInfoVoList(inStockInfoVoList);
                    }
                    if (currTask1.getStatus() == 1) {
                        nowStep = currTask1.getTargetPosition();
                    } else if (currTask1.getIsCache() == 1) {
                        nowStep = currTask.getLocationName();
                    }
                    if (currTaskInfoVo.getInStockInfoVoList() != null) {
                        if (currTaskInfoVo.getCurrTaskVoList().size() > 0) {

                            nowStep = currTaskInfoVo.getInStockInfoVoList().get(0).getLocationName();
                        }
                    }
                    allStep = allStep.concat("->").concat(currTask1.getTargetPosition());
                }
                //
                currTaskInfoVo.setAllStep(allStep);
                currTaskInfoVo.setNowStep(nowStep);
                currTaskInfoVo.setAutoExeTaskId(currTask.getAutoExeTaskId());
            } else {
                currTaskInfoVo.setAllStep(currTask.getTargetPosition());
                if (currTask.getStatus() == 1) {
                    currTaskInfoVo.setNowStep(currTask.getTargetPosition());
                } else if (currTask.getIsCache() == 1) {
                    currTaskInfoVo.setNowStep(currTask.getLocationName());
                }
                currTaskInfoVo.setAutoExeTaskId(currTask.getId());
                InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.TASK_ID, currTask.getId()));
                if (inStockInfo != null) {
                    List<InStockInfoVo> inStockInfoVoList = new ArrayList<>();
                    InStockInfoVo inStockInfoVo = InStockInfoVo.builder().build();
                    BeanUtils.copyProperties(inStockInfo, inStockInfoVo);
                    inStockInfoVoList.add(inStockInfoVo);
                    currTaskInfoVo.setInStockInfoVoList(inStockInfoVoList);
                }
            }
            //currTaskDet
            List<CurrTaskDet> currTaskDetList = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask.getId()));
            currTaskInfoVo.setCurrTaskDetVoList(com.mt.wms.core.utils.BeanUtils.copyList(currTaskDetList, CurrTaskDetVo.class));
            for (CurrTaskDetVo currTaskDetVo : currTaskInfoVo.getCurrTaskDetVoList()
            ) {
                String idenCardNum = currTaskDetVo.getIdenCardNum();
                OrderInfo orderInfo = orderInfoServiceBiz.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, idenCardNum));
                currTaskDetVo.setCustomer(orderInfo.getCustomerName());
                currTaskDetVo.setCraftIll(orderInfo.getCraftIll());
                currTaskDetVo.setProductModel(orderInfo.getProductModel());
            }
            list.add(currTaskInfoVo);
        }
        list = list.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(CurrTaskInfoVo::getAutoExeTaskId))), ArrayList::new));

        return successful(list);
    }

    //获取未成功报工apms的任务,判定标准:没有生产单号,状态为完成
    @Override
    public R<List<CurrTaskInfoVo>> currentTaskInfoPageForNotToApms(CurrTaskQueryParam param) {
        QueryWrapper<CurrTask> wrapper = new QueryWrapper<>();
        wrapper.eq(CurrTask.VALID, 1);
        wrapper.in(CurrTask.STATUS, 2);
        wrapper.isNull(CurrTask.SHEET_NO);
        wrapper.between(CurrTask.CREATE_TIME, "2022-11-19 05:14:11", LocalDateTime.now());
        List<CurrTask> currTaskList = currTaskServiceBiz.list(wrapper);
        List<CurrTaskInfoVo> list = new ArrayList<>();
        for (CurrTask currTask : currTaskList
        ) {
            CurrTaskInfoVo currTaskInfoVo = CurrTaskInfoVo.builder().build();
            //currTask
            if (currTask.getIsAuto() == 1) {


                currTaskInfoVo.setAutoExeTaskId(currTask.getAutoExeTaskId());
                List<CurrTask> currTasks = currTaskServiceBiz
                        .list(new QueryWrapper<CurrTask>()
                                .eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId())
                                .orderByAsc(CurrTask.ID));
                List<CurrTaskVo> currTaskVos = com.mt.wms.core.utils.BeanUtils.copyList(currTasks, CurrTaskVo.class);
                currTaskInfoVo.setCurrTaskVoList(currTaskVos);
                String allStep = "";
                for (CurrTask currTask1 : currTasks
                ) {

                    allStep = allStep.concat("->").concat(currTask1.getTargetPosition());
                }
                currTaskInfoVo.setAllStep(allStep);
            } else {
                currTaskInfoVo.setAllStep(currTask.getTargetPosition());
            }
            //currTaskDet
            List<CurrTaskDet> currTaskDetList = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask.getId()));
            currTaskInfoVo.setCurrTaskDetVoList(com.mt.wms.core.utils.BeanUtils.copyList(currTaskDetList, CurrTaskDetVo.class));
            for (CurrTaskDetVo currTaskDetVo : currTaskInfoVo.getCurrTaskDetVoList()
            ) {
                String idenCardNum = currTaskDetVo.getIdenCardNum();
                OrderInfo orderInfo = orderInfoServiceBiz.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, idenCardNum));
                currTaskDetVo.setCustomer(orderInfo.getCustomerName());
                currTaskDetVo.setCraftIll(orderInfo.getCraftIll());
                currTaskDetVo.setProductModel(orderInfo.getProductModel());
            }
            list.add(currTaskInfoVo);
        }
        list = list.stream().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(CurrTaskInfoVo::getAutoExeTaskId))), ArrayList::new));

        return successful(list);
    }

    @Override
    public R<String> oneStepToApms(CurrTaskInfoVo currTaskInfoVo) {
        List<CurrTaskDetVo> currTaskDetVoList = currTaskInfoVo.getCurrTaskDetVoList();
        List<CurrTaskVo> currTaskVoList = currTaskInfoVo.getCurrTaskVoList();
        //报工apms
        List<CurrTaskDet> currTaskDetList = com.mt.wms.core.utils.BeanUtils.copyList(currTaskDetVoList, CurrTaskDet.class);
        List<CurrTask> currTasks = com.mt.wms.core.utils.BeanUtils.copyList(currTaskVoList, CurrTask.class);
        for (CurrTask currTask1 : currTasks
        ) {
            if (currTask1.getIsAuto() == 1) {
                AutoExeTask autoExeTask = autoExeTaskServiceBiz.getById(currTaskInfoVo.getAutoExeTaskId());
                KilnInfo kilnInfo = kilnInfoServiceBiz.getById(currTask1.getKilnId());
                //加工炉为BMA或BMB时上报apms,其他的不需要上报
                if (kilnInfo.getType() == 1 || kilnInfo.getType() == 3) {
                    if (currTask1.getStoveCode() == null) {
                        return failed("该任务没有正常进炉加工生成炉号,不能上报apms!");
                    }
                    //通知apms创建一个生产单
                    ApmsCreateProcessSheet apmsCreateSheet = new ApmsCreateProcessSheet();
                    apmsCreateSheet.setWorkShopCode("BM");
                    apmsCreateSheet.setStartTime(currTask1.getBeginTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                    apmsCreateSheet.setTechCode(currTask1.getCraftCodeId().toString());
                    //fixme 暂时没有正式user，使用测试账户
                    apmsCreateSheet.setStartUser("QJJP03");
                    ArrayList<CreateItem> createItems = new ArrayList<>();
                    currTaskDetList.forEach(e -> {
                        CreateItem createItem = new CreateItem();
                        createItem.setItemCode(e.getIdenCardNum());
                        createItem.setQuantity(e.getQuantity().intValue());
                        BigDecimal b = new BigDecimal(String.valueOf(e.getWeight()));
                        createItem.setWeight(b.doubleValue());
                        createItems.add(createItem);
                    });
                    //apmsCreateSheet.setStoveCode(code);
                    if (currTask1.getStoveCode() != null) {
                        apmsCreateSheet.setStoveCode(currTask1.getStoveCode());
                    }
                    apmsCreateSheet.setItems(createItems);
                    logger.info(currTask1.getId() + "开始创建apms生产单号-手动上报");
                    R<ApmsCreateProcessSheetVo> processSheet = apmsControl.createProcessSheet(apmsCreateSheet);
                    if (!processSheet.getData().getSuccess()) {
                        logger.info(currTask1.getId() + "创建失败，APMS报错：" + processSheet.getData().getMsg());
                        return failed(processSheet.getData().getMsg());
                    } else {
                        logger.info(currTask1.getId() + "创建apms生产单号成功");
                        currTask1.setSheetNo(processSheet.getData().getSheetNo());
                        currTaskServiceBiz.updateById(currTask1);
                        //多步骤任务除了第一步写入sheetNo
                        if (currTask1.getIsAuto() == 1) {
                            CurrTask secondCurrTask = currTaskServiceBiz.getById(autoExeTask.getTTaskId());
                            secondCurrTask.setSheetNo(currTask1.getSheetNo());
                            currTaskServiceBiz.updateById(secondCurrTask);
                            //三步任务
                            if (autoExeTask.getProcessFlowType() == 2) {
                                CurrTask threeCurrTask = currTaskServiceBiz.getById(autoExeTask.getThTaskId());
                                threeCurrTask.setSheetNo(currTask1.getSheetNo());
                                currTaskServiceBiz.updateById(threeCurrTask);
                            }
                        }
                        return successful("上报APMS成功");
                    }
                }
            } else {
                //通知apms创建一个生产单
                ApmsCreateProcessSheet apmsCreateSheet = new ApmsCreateProcessSheet();
                apmsCreateSheet.setWorkShopCode("BM");
                apmsCreateSheet.setStartTime(currTask1.getBeginTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
                apmsCreateSheet.setTechCode(currTask1.getCraftCodeId().toString());
                //fixme 暂时没有正式user，使用测试账户
                apmsCreateSheet.setStartUser("QJJP03");
                ArrayList<CreateItem> createItems = new ArrayList<>();
                currTaskDetList.forEach(e -> {
                    CreateItem createItem = new CreateItem();
                    createItem.setItemCode(e.getIdenCardNum());
                    createItem.setQuantity(e.getQuantity().intValue());
                    BigDecimal b = new BigDecimal(String.valueOf(e.getWeight()));
                    createItem.setWeight(b.doubleValue());
                    createItems.add(createItem);
                });
                //apmsCreateSheet.setStoveCode(code);
                if (currTask1.getStoveCode() != null) {
                    apmsCreateSheet.setStoveCode(currTask1.getStoveCode());
                }
                apmsCreateSheet.setItems(createItems);
                logger.info(currTask1.getId() + "开始创建apms生产单号-手动上报");
                R<ApmsCreateProcessSheetVo> processSheet = apmsControl.createProcessSheet(apmsCreateSheet);
                if (!processSheet.getData().getSuccess()) {
                    logger.info(currTask1.getId() + "创建失败，APMS报错：" + processSheet.getData().getMsg());
                    return failed(processSheet.getData().getMsg());
                } else {
                    logger.info(currTask1.getId() + "创建apms生产单号成功");
                    currTask1.setSheetNo(processSheet.getData().getSheetNo());
                    currTaskServiceBiz.updateById(currTask1);
                }
            }
        }
        return null;
    }

    @Override
    public R<String> updateCurrTaskStatus(CurrTaskParam param) {
        CurrTask currTask = currTaskServiceBiz.getById(param.getId());
        Integer status = param.getStatus();
        Long kilnId = param.getKilnId();
        //修改任务状态
        if (!status.equals(currTask.getStatus())) {
            //修改为加工中,多步骤任务修改为加工中,查询缓存区,查询前置任务状态,并修改
            if (status == 1) {
                currTask.setStatus(1);
                currTask.setIsIn(1);
                currTask.setUpdaterId(99);
                currTask.setUpdateTime(LocalDateTime.now());
                currTask.setBeginTime(LocalDateTime.now());
                //生产炉号
                createStoveCode(currTask);
                InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.TASK_ID, currTask.getId()));
                if (inStockInfo != null) {
                    //更新库位状态为未占用
                    Location location = locationServiceBiz.getById(inStockInfo.getLocationId());
                    location.setStatus(0);
                    locationServiceBiz.updateById(location);
                    //删除库位缓存
                    inStockInfoServiceBiz.removeById(inStockInfo);
                }
                if (currTask.getIsAuto() == 1) {
                    AutoExeTask autoExeTask = autoExeTaskServiceBiz.getById(currTask.getAutoExeTaskId());
                    if (currTask.getId().equals(autoExeTask.getFTaskId())) {
                        //该任务为自动任务第一条,是否有sheetNo,若无上报apms
                        if (currTask.getSheetNo() == null) {
                            try {
                                apmsCreateProcess(currTask);
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                        }
                    }
                    if (currTask.getId().equals(autoExeTask.getTTaskId())) {
                        //该任务为自动任务第二条,修改自动任务第一条的状态
                        CurrTask currTask1 = currTaskServiceBiz.getById(autoExeTask.getFTaskId());
                        currTask1.setStatus(2);
                        currTask1.setIsIn(2);
                        //99代表该条记录手动修改过
                        currTask1.setUpdaterId(99);
                        currTask1.setUpdateTime(LocalDateTime.now());
                        currTaskServiceBiz.updateById(currTask1);
                        apmsEndProcess(currTask1);
                        apmsStartProcess(currTask);
                    } else if (currTask.getId().equals(autoExeTask.getThTaskId())) {
                        //该任务为自动任务第三条,修改自动任务第二条的状态
                        CurrTask currTask2 = currTaskServiceBiz.getById(autoExeTask.getFTaskId());
                        currTask2.setStatus(2);
                        currTask2.setIsIn(2);
                        //99代表该条记录手动修改过
                        currTask2.setUpdaterId(99);
                        currTask2.setUpdateTime(LocalDateTime.now());
                        currTaskServiceBiz.updateById(currTask2);
                        apmsEndProcess(currTask2);
                        apmsStartProcess(currTask);
                    }
                }

                currTaskServiceBiz.updateById(currTask);
                logger.info("修改任务" + currTask.getId() + "状态为加工中成功");
            }
            //修改为加工完成,查询缓存,并修改
            if (status == 2) {
                InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.TASK_ID, currTask.getId()));
                if (inStockInfo != null) {
                    //更新库位状态为未占用
                    Location location = locationServiceBiz.getById(inStockInfo.getLocationId());
                    location.setStatus(0);
                    locationServiceBiz.updateById(location);
                    //删除库位缓存
                    inStockInfoServiceBiz.removeById(inStockInfo);
                }
                currTask.setStatus(1);
                currTask.setCreatorId(99);
                currTask.setBeginTime(LocalDateTime.now());
                currTaskServiceBiz.updateById(currTask);
                logger.info("修改任务" + currTask.getId() + "状态为加工完成成功");
            }
            currTask.setStatus(status);
            currTask.setCreatorId(99);
            currTask.setBeginTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            logger.info("修改任务" + currTask.getId() + "状态为未加工成功");
        }
        //修改目标加工炉
        if (!kilnId.equals(currTask.getKilnId())) {
            KilnInfo kiln = kilnInfoServiceBiz.getById(kilnId);
            //修改目标加工炉,修改终点
            currTask.setKilnId(kilnId);
            currTask.setTargetPosition(kiln.getCode());
            currTask.setUpdaterId(99);
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.TASK_ID, currTask.getId()));
            //是否存在缓存
            if (inStockInfo != null) {
                //修改缓存后目标加工炉
                inStockInfo.setKilnId(kilnId);
                inStockInfo.setKilnName(kiln.getKilnName());
                inStockInfo.setUpdaterId(99);
                inStockInfo.setUpdateTime(LocalDateTime.now());
                inStockInfoServiceBiz.updateById(inStockInfo);
            }
            //任务为多步骤任务
            if (currTask.getIsAuto() == 1) {
                AutoExeTask autoExeTask = autoExeTaskServiceBiz.getById(currTask.getAutoExeTaskId());
                //任务为多步骤任务第一步
                if (currTask.getId().equals(autoExeTask.getFTaskId())) {
                    //修改多步骤第二步的起点
                    CurrTask currTask2 = currTaskServiceBiz.getById(autoExeTask.getTTaskId());
                    currTask2.setStartPosition(kiln.getCode());
                    currTask2.setUpdateTime(LocalDateTime.now());
                    currTask2.setUpdaterId(99);
                    currTaskServiceBiz.updateById(currTask2);
                } else if (currTask.getId().equals(autoExeTask.getTTaskId())) {
                    //任务为多步骤任务第二步且有第三步,修改第三步起点
                    if (autoExeTask.getProcessFlowType() == 2) {
                        CurrTask currTask3 = currTaskServiceBiz.getById(autoExeTask.getThTaskId());
                        currTask3.setStartPosition(kiln.getCode());
                        currTask3.setUpdateTime(LocalDateTime.now());
                        currTask3.setUpdaterId(99);
                        currTaskServiceBiz.updateById(currTask3);
                    }
                }
            }
        }
        return successful("操作成功!");
    }


    @Override
    public R<String> completeTask(IdParam param) {
        CurrTask byId = currTaskServiceBiz.getById(param.getId());
        TaskHis taskHis = new TaskHis();
        BeanUtils.copyProperties(byId, taskHis);
        taskHis.setStatus(2);
        taskHis.setId(null);
        taskHis.setInterCode(IDGenerator.gen("RWHIS", IDGenerator.PATTERN_YYYYMMDDHHMM, 2, "TASKHIS_CODE"));
        setCommonField(taskHis);
        //将主任务放入到历史表，清空id，防止重复
        R<IdVo> save = taskHisService.save(taskHis);
        byId.setStatus(2);
        currTaskServiceBiz.removeById(byId);
        List<CurrTaskDet> taskDetailsByMainId = currTaskDetService.getTaskDetailsByMainId(IdParam.builder().id(byId.getId()).build());
        List<TaskDetHis> taskDetHis = com.mt.wms.core.utils.BeanUtils.copyList(taskDetailsByMainId, TaskDetHis.class);
        taskDetHis.forEach(detHis -> {
            detHis.setCurrTaskId(save.getData().getId());
        });
        //将详细任务信息放入详细历史表
        taskDetHisService.saveList(taskDetHis);
        return successful("执行成功。");
    }

    @Override
    public R<String> deleteBySheetNo(String sheetNo) {
        //一个生产单可能对应多个currTask,删除的时候查询是否对应多个，如果对应多个，自动循环删除所有currTask,当删除最后一条currTask时上报apms
        List<CurrTask> currTaskList = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.SHEET_NO, sheetNo));
        if (currTaskList.size() > 1) {
            int i = 0;
            for (CurrTask currTask : currTaskList
            ) {
                i++;
                if (currTask.getStatus() == 0) {
                    if (i == currTaskList.size()) {
                        R<ApmsEndProcessVo> apmsStartProcessR = apmsControl.deleteBySheetNo(sheetNo);
                        if (!apmsStartProcessR.getData().getSuccess()) {
                            return failed(apmsStartProcessR.getData().getMsg());
                        }
                    }
                    Long currTaskId = currTask.getId();
                    if (currTask.getIsAuto() == 1) {
                        List<CurrTask> currTaskList2 = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
                        if (currTaskList2.size() == 1) {
                            autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
                        }
                    }
                    currTaskServiceBiz.removeById(currTaskId);
                    currTaskDetService.deleteByCurrTaskId(currTaskId);

                }
                if (currTask.getStatus() == 2) {
                    return failed("无法删除，任务已加工完成。");
                }
                if (currTask.getStatus() == 1) {
                    return failed("无法删除，任务正在执行中。");
                }
            }
            return successful(0, "APMS生产单删除成功。", "");
        } else {
            for (CurrTask currTask : currTaskList
            ) {
                if (currTask.getStatus() == 0) {
                    R<ApmsEndProcessVo> apmsStartProcessR = apmsControl.deleteBySheetNo(sheetNo);
                    if (apmsStartProcessR.getData().getSuccess()) {
                        Long currTaskId = currTask.getId();
                        if (currTask.getIsAuto() == 1) {
                            List<CurrTask> currTaskList2 = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
                            if (currTaskList2.size() == 1) {
                                autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
                            }
                        }
                        currTaskServiceBiz.removeById(currTaskId);
                        currTaskDetService.deleteByCurrTaskId(currTaskId);
                        if (currTask.getIsAuto() == 1) {
                            List<CurrTask> currTaskList2 = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
                            if (currTaskList2.size() == 0) {
                                autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
                            }
                        }
                        return successful(0, "APMS生产单删除成功。", "");
                    } else {
                        return failed(apmsStartProcessR.getData().getMsg());
                    }
                }
                if (currTask.getStatus() == 2) {
                    return failed("无法删除，任务已加工完成。");
                }
                if (currTask.getStatus() == 1) {
                    return failed("无法删除，任务正在执行中。");
                }

            }
        }
        return failed("根据生产单号删除任务失败！");

    }

    @Override
    public R<String> deleteById(IdParam idParam) {
        CurrTask currTask = currTaskServiceBiz.getById(idParam.getId());
        if (currTask.getIsAuto() == 1) {
            List<CurrTask> currTaskList = currTaskServiceBiz.list(new QueryWrapper<CurrTask>().eq(CurrTask.AUTO_EXE_TASK_ID, currTask.getAutoExeTaskId()));
            for (CurrTask currTask1 : currTaskList
            ) {
                if (currTask1.getStatus() != 0) {
                    return R.failed("该自动任务已有步骤在执行中或执行完成，不能删除！");
                }
            }
            //能删除
            autoExeTaskServiceBiz.removeById(currTask.getAutoExeTaskId());
            for (CurrTask currTask1 : currTaskList
            ) {
                InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.TASK_ID, currTask1.getId()));
                if (inStockInfo != null) {
                    Location location = locationServiceBiz.getById(inStockInfo.getLocationId());
                    location.setStatus(0);
                    locationServiceBiz.updateById(location);
                    inStockInfoServiceBiz.removeById(inStockInfo);
                }
                currTaskDetServiceBiz.remove(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask1.getId()));
                currTaskServiceBiz.removeById(currTask1);
            }
            return R.ok(idParam.getId().toString());
        } else {
            currTaskDetServiceBiz.remove(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask.getId()));
            currTaskServiceBiz.removeById(idParam);
            return R.ok(idParam.getId().toString());
        }

    }

    @Override
    public R<PageVo<CurrTaskMainQueryVo>> currentTaskMainPage(BasePageParam param) {
        QueryWrapper<CurrTask> currTaskQueryWrapper = new QueryWrapper<>();
        currTaskQueryWrapper.eq(CurrTask.STATUS, 1);
        currTaskQueryWrapper.eq(CurrTask.VALID, 1);
        Page<CurrTask> page = currTaskServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), currTaskQueryWrapper);
        PageVo<CurrTaskMainQueryVo> pageVo = new PageVo<>(page, CurrTaskMainQueryVo.class);
        Assert.notNull("无执行中的任务！", pageVo.getRecords());
        pageVo.getRecords().forEach(e ->
        {
            //根据当前主任务查详细任务
            List<CurrTaskDet> taskDetailsByMainId = currTaskDetService.getTaskDetailsByMainId(IdParam.builder().id(e.getId()).build());
            ArrayList<String> materialDes = new ArrayList<>();
            ArrayList<String> iden = new ArrayList<>();
            //写入材料名与标识卡号
            taskDetailsByMainId.forEach(a -> {
                materialDes.add(a.getMaterialDes());
                iden.add(a.getIdenCardNum());
            });
            e.setMaterials(materialDes);
            e.setIdens(iden);
            ArrayList<String> customers = new ArrayList<>();
            //根据标识卡号查订单的客户名并写入
            e.getIdens().forEach(id -> {
                        OrderInfo one = orderInfoServiceBiz.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, id));
                        customers.add(one.getCustomerName());
                    }
            );
            e.setCustomers(customers);
            e.setKilnName(e.getTargetPosition());
            //e.setLocationName(locationServiceBiz.getById(e.getLocationId()).getLocationName());
        });
        return successful(pageVo);
    }

    @Override
    public R runTask(Long taskId) throws InterruptedException, IOException {
        CurrTask currTask = currTaskServiceBiz.getById(taskId);
        if (currTask.getStatus() != 0) {
            return R.failed("该任务状态不为未执行,无法执行!");
        }
        //是否为自动任务
        if (currTask.getIsAuto() == 1) {
            List<AutoExeTask> exeTaskList = autoExeTaskServiceBiz.list(new QueryWrapper<AutoExeTask>().eq(AutoExeTask.F_TASK_ID, taskId));
            if (exeTaskList.size() == 0) {
                return R.failed("该任务为多步骤加工任务且该步骤不是第一步，请选择第一步骤任务执行！");
            }
        }
        return runTaskUtils.runTaskForInKiln(taskId);
    }

    @Override
    public R<String> createAutoTask(AutoCurrTaskParam autoCurrTaskParam) {
        return null;
    }

    /**
     * 传入任务id,查询任务炉子，然后根据炉子查询等待列表，然后时间置为早
     *
     * @param idParam
     * @return
     */
    @Override
    public R<String> top(IdParam idParam) {
        CurrTask currTask = currTaskServiceBiz.getById(idParam.getId());
        currTask.getIsCache();
        if (currTask.getIsCache() == 0) {
            return R.failed("当前任务未进入等待队列，不能置顶");
        }
        Long kilnId = currTask.getKilnId();
        List<InStockInfo> inStockInfoList = inStockInfoServiceBiz.list(new QueryWrapper<InStockInfo>()
                .eq(InStockInfo.KILN_ID, kilnId)
                .eq(InStockInfo.TYPE, 1)
                .eq(InStockInfo.STATUS, 2)
                .orderByAsc(InStockInfo.IN_TIME));

        if (inStockInfoList.size() < 2) {
            return R.failed("当前加工炉等待队列只有一个任务，无需置顶");
        }
        LocalDateTime inTime = null;
        inTime = inStockInfoList.get(0).getInTime();
        InStockInfo inStockInfo = inStockInfoServiceBiz.getOne(new QueryWrapper<InStockInfo>().eq(InStockInfo.LOCATION_ID, currTask.getLocationId()));
        inStockInfo.setInTime(inTime.minusHours(1));
        inStockInfoServiceBiz.updateById(inStockInfo);
        return R.ok("当前任务置顶成功", "");
    }

    /**
     * 上报apms创建生产单并开始处理
     *
     * @param currTask 任务
     * @return 成功与否
     * @throws IOException
     */
    private Boolean apmsCreateProcess(CurrTask currTask) throws IOException {
        List<CurrTaskDet> currTaskDetList = currTaskDetServiceBiz.list(new QueryWrapper<CurrTaskDet>().eq(CurrTaskDet.CURR_TASK_ID, currTask.getId()));
        String code = kilnInfoServiceBiz.getById(currTask.getKilnId()).getCode();
        //通知apms创建一个生产单
        ApmsCreateProcessSheet apmsCreateSheet = new ApmsCreateProcessSheet();
        apmsCreateSheet.setWorkShopCode("BM");
        apmsCreateSheet.setStartTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsCreateSheet.setTechCode(currTask.getCraftCodeId().toString());
        //fixme 暂时没有正式user，使用测试账户
        apmsCreateSheet.setStartUser("QJJP03");
        ArrayList<CreateItem> createItems = new ArrayList<>();
        currTaskDetList.forEach(e -> {
            CreateItem createItem = new CreateItem();
            createItem.setItemCode(e.getIdenCardNum());
            createItem.setQuantity(e.getQuantity().intValue());
            createItem.setWeight(e.getWeight().doubleValue());
            createItems.add(createItem);
        });
        apmsCreateSheet.setStoveCode(code);
        apmsCreateSheet.setItems(createItems);
        logger.info(currTask.getId() + "开始创建apms生产单号");
        R<ApmsCreateProcessSheetVo> processSheet = apmsControl.createProcessSheet(apmsCreateSheet);
        if (!processSheet.getData().getSuccess()) {
            logger.info(currTask.getId() + "创建失败，APMS报错：" + processSheet.getData().getMsg());
            webSocketServer.sendtoUser("创建失败，APMS报错：" + processSheet.getData().getMsg(), "1");
            return false;
        } else {
            logger.info(currTask.getId() + "创建apms生产单号成功");
            currTask.setSheetNo(processSheet.getData().getSheetNo());
            currTaskServiceBiz.updateById(currTask);
            //多步骤任务除了第一步写入sheetNo
            if (currTask.getIsAuto() == 1) {
                AutoExeTask autoExeTask = autoExeTaskServiceBiz.getById(currTask.getAutoExeTaskId());
                CurrTask secondCurrTask = currTaskServiceBiz.getById(autoExeTask.getTTaskId());
                secondCurrTask.setSheetNo(currTask.getSheetNo());
                currTaskServiceBiz.updateById(secondCurrTask);
                //三步任务
                if (autoExeTask.getProcessFlowType() == 2) {
                    CurrTask threeCurrTask = currTaskServiceBiz.getById(autoExeTask.getThTaskId());
                    threeCurrTask.setSheetNo(currTask.getSheetNo());
                    currTaskServiceBiz.updateById(threeCurrTask);
                }
            }
            return true;
        }
    }
    /**
     * 上报apms开始加工
     *
     * @param currTask
     * @return
     */
    private Boolean apmsStartProcess(CurrTask currTask) {
        String code = kilnInfoServiceBiz.getById(currTask.getKilnId()).getCode();
        ApmsStartProcess apmsStartProcess = new ApmsStartProcess();
        apmsStartProcess.setStartUser("QJJP03");
        apmsStartProcess.setStartTime(LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsStartProcess.setTechName(currTask.getCraftCodeId().toString());
        apmsStartProcess.setSheetNo(currTask.getSheetNo());
        if (currTask.getStoveCode() != null) {
            apmsStartProcess.setStoveCode(currTask.getStoveCode());
        }

        R<ApmsStartProcess> apmsStartProcessR = apmsControl.startProcess(apmsStartProcess);
        if (apmsStartProcessR.getCode() != 0) {
            logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单开始生产上报apms失败");
            return false;
        }
        logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单开始生产上报apms成功");
        return true;
    }

    /**
     * 上报apms-end
     *
     * @param currTask
     * @return
     */
    private Boolean apmsEndProcess(CurrTask currTask) {
        ApmsEndProcess apmsEndProcess = new ApmsEndProcess();
        apmsEndProcess.setEndTime(currTask.getEndTime().format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        apmsEndProcess.setEndUser("QJJP03");
        apmsEndProcess.setSheetNo(currTask.getSheetNo());
        R<ApmsEndProcessVo> apmsEndProcessVoR = apmsControl.endProcess(apmsEndProcess);
        if (apmsEndProcessVoR.getCode() != 0) {
            logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单完成处理上报apms失败");
            return false;
        }
        logger.info("任务号" + currTask.getTaskCode() + "生产单号：" + currTask.getSheetNo() + "生产单完成处理上报apms成功");
        return true;
    }
    /**
     * 生成炉号
     * 多步骤任务除了第一步为清洗炉外,每步都生成炉号
     * 单步骤任务都生成炉号
     */
    private void createStoveCode(CurrTask currTask) {
        //单步骤任务,直接生成炉号
        if (currTask.getIsAuto() == 0) {
            String stoveCode = StoveCodeUtils.getStoveCode(currTask.getTargetPosition(), currTask.getPlcValue());
            currTask.setStoveCode(stoveCode);
            currTaskServiceBiz.updateById(currTask);
            return;
        }
        //多步骤任务,只有BMA,BMB步骤生成炉号
        else {
            //获取加工炉
            KilnInfo kilnInfo = kilnInfoServiceBiz.getById(currTask.getKilnId());
            //只有BMA,BMB生成炉号
            if (kilnInfo.getType() == 1 || kilnInfo.getType() == 3) {
                String stoveCode = StoveCodeUtils.getStoveCode(currTask.getTargetPosition(), currTask.getPlcValue());
                currTask.setStoveCode(stoveCode);
                currTaskServiceBiz.updateById(currTask);
            } else {
                logger.info(currTask.getId() + "任务为多步骤任务但该步骤不生成洗炉炉号.");
            }
        }
    }
}
