package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.basic.params.KilnInfoQueryParam;
import com.mt.wms.basic.params.VehicleQueryParam;
import com.mt.wms.basic.service.KilnInfoService;
import com.mt.wms.basic.service.VehicleService;
import com.mt.wms.basic.vo.KilnInfoVo;
import com.mt.wms.basic.vo.VehicleVo;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.OrderInfo;
import com.mt.wms.core.dal.entity.OrderInfoHis;
import com.mt.wms.core.dal.service.OrderInfoHisServiceBiz;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.OrderInfoBasePageParam;
import com.mt.wms.empty.params.OrderInfoTaskDetParam;
import com.mt.wms.empty.service.OrderInfoHisService;
import com.mt.wms.empty.service.TaskDetHisService;
import com.mt.wms.empty.service.TaskHisService;
import com.mt.wms.empty.vo.CurrTaskVo;
import com.mt.wms.empty.vo.OrderInfoTaskDetVo;
import com.mt.wms.empty.vo.OrderInfoVo;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class OrderInfoHisServiceImpl extends BaseService implements OrderInfoHisService {

    @Autowired
    OrderInfoHisServiceBiz orderInfoHisServiceBiz;

    @Autowired
    TaskDetHisService taskDetHisService;

    @Autowired
    TaskHisService taskHisService;

    @Autowired
    VehicleService vehicleService;

    @Autowired
    KilnInfoService kilnInfoService;

    @Override
    public R<IdVo> save(OrderInfo orderInfo) {
        OrderInfoHis orderInfoHis = new OrderInfoHis();
        BeanUtils.copyProperties(orderInfo, orderInfoHis);
        //设置任务状态位完成
        orderInfoHis.setStatus(2);
        orderInfoHisServiceBiz.save(orderInfoHis);
        return null;
    }

    @Override
    public R<PageVo<OrderInfoVo>> page(OrderInfoBasePageParam param) {
        QueryWrapper<OrderInfoHis> wrapper = new QueryWrapper<>();
        wrapper.like(StringUtils.isNotBlank(param.getIdenCardNum()), OrderInfo.IDEN_CARD_NUM, param.getIdenCardNum())
                .like(StringUtils.isNotBlank(param.getOrderNo()), OrderInfo.ORDER_NO, param.getOrderNo())
                .eq(OrderInfo.VALID, 1)
                .orderByDesc(OrderInfo.CREATE_TIME);
        Page<OrderInfoHis> page = orderInfoHisServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), wrapper);
        return successful(new PageVo<>(page, OrderInfoVo.class));
    }

    @Override
    public R<OrderInfoVo> getOne(IdParam idParam) {
        OrderInfoHis byId = orderInfoHisServiceBiz.getById(idParam.getId());
        OrderInfoVo build = OrderInfoVo.builder().build();
        BeanUtils.copyProperties(byId, build);
        return successful(build);
    }

    @Override
    public R<PageVo<OrderInfoTaskDetVo>> getTaskInfo(OrderInfoTaskDetParam param) {
        //从历史任务详细中获取该标识卡和订单的所有订单
        PageVo<OrderInfoTaskDetVo> currOrderCurrTask = taskDetHisService.getHisOrderHisTask(param);
        Assert.notNull("该订单下无历史任务！", currOrderCurrTask.getRecords());
        //获取车辆信息
        R<List<VehicleVo>> vehicle = vehicleService.list(new VehicleQueryParam());
        Map<Long, List<VehicleVo>> collect = vehicle.getData().stream().collect(Collectors.groupingBy(VehicleVo::getId));
        //获取工业炉信息
        R<List<KilnInfoVo>> kiln = kilnInfoService.list(new KilnInfoQueryParam());
        Map<Long, List<KilnInfoVo>> collect2 = kiln.getData().stream().collect(Collectors.groupingBy(KilnInfoVo::getId));
        currOrderCurrTask.getRecords().forEach(orderInfo ->
                {
                    //获取对应的任务
                    CurrTaskVo currTask = taskHisService.getById(IdParam.builder().id(orderInfo.getCurrTaskId()).build());
                    orderInfo.setABeginTime(currTask.getBeginTime());
                    orderInfo.setAEndTime(currTask.getEndTime());
                    orderInfo.setTaskType(currTask.getTaskType());
                    orderInfo.setStatus(currTask.getStatus());
                    orderInfo.setKilnId(currTask.getKilnId());
                    orderInfo.setStartPosition(currTask.getStartPosition());
                    orderInfo.setTargetPosition(currTask.getTargetPosition());
                    orderInfo.setVehicleId(currTask.getVehicleId());
                    //获取工业炉和车辆的名称信息
                    if (orderInfo.getKilnId() != null)
                    {
                        orderInfo.setKilnName(collect2.get(orderInfo.getKilnId()).get(0).getKilnName());
                    }
                    if (orderInfo.getVehicleId() != null)
                    {
                        orderInfo.setVehicleName(collect.get(orderInfo.getVehicleId()).get(0).getVehicleName());
                    }
                }
        );
        return successful(currOrderCurrTask);
    }
}
