/*
 * Copyright (c) 2018.
 * http://www.ulabcare.com
 */

package com.mt.wms.upms.service;

import com.mt.wms.core.params.EnabledParam;
import com.mt.wms.core.params.IdListParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdListVo;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.upms.params.ResourceParam;
import com.mt.wms.upms.params.ResourceQueryParam;
import com.mt.wms.upms.vo.ResourceVo;

import java.util.List;

/**
 * 资源服务接口
 *
 * @author jiff
 * @date 2018/11/14
 * @since 1.0
 */
public interface ResourceService {

    /**
     * 获取资源
     *
     * @param idParam 主键参数
     * @return 资源
     */
    R<ResourceVo> get(IdParam idParam);

    /**
     * 获取资源列表
     *
     * @param resourceQueryParam 资源查询参数
     * @return 资源列表
     */
    R<List<ResourceVo>> list(ResourceQueryParam resourceQueryParam);

    /**
     * 获取资源分页列表
     *
     * @param resourceQueryParam 资源查询参数
     * @return 资源分页列表
     */
    R<PageVo<ResourceVo>> page(ResourceQueryParam resourceQueryParam);

    /**
     * 新增资源
     *
     * @param resourceParam 资源参数
     * @return 主键
     */
    R<IdVo> add(ResourceParam resourceParam);

    /**
     * 更新资源
     *
     * @param resourceParam 资源参数
     * @return 主键
     */
    R<IdVo> update(ResourceParam resourceParam);

    /**
     * 启用停用
     *
     * @param enabledParam 启停参数
     * @return 主键
     */
    R<IdVo> enabled(EnabledParam enabledParam);

    /**
     * 删除资源
     *
     * @param idParam 主键参数
     * @return 主键
     */
    R<IdVo> delete(IdParam idParam);

    /**
     * 批量删除资源
     *
     * @param idListParam 主键列表参数
     * @return 主键列表
     */
    R<IdListVo> delete(IdListParam idListParam);

}
